# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: contrib/envoy/extensions/compression/qatzstd/compressor/v3alpha/qatzstd.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'contrib/envoy/extensions/compression/qatzstd/compressor/v3alpha/qatzstd.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nMcontrib/envoy/extensions/compression/qatzstd/compressor/v3alpha/qatzstd.proto\x12\x37\x65nvoy.extensions.compression.qatzstd.compressor.v3alpha\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xf9\x03\n\x07Qatzstd\x12\x42\n\x11\x63ompression_level\x18\x01 \x01(\x0b\x32\x1c.google.protobuf.UInt32ValueB\t\xfa\x42\x06*\x04\x18\x16(\x01\x12\x17\n\x0f\x65nable_checksum\x18\x02 \x01(\x08\x12\x65\n\x08strategy\x18\x03 \x01(\x0e\x32I.envoy.extensions.compression.qatzstd.compressor.v3alpha.Qatzstd.StrategyB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12>\n\nchunk_size\x18\x05 \x01(\x0b\x32\x1c.google.protobuf.UInt32ValueB\x0c\xfa\x42\t*\x07\x18\x80\x80\x04(\x80 \x12\x17\n\x0f\x65nable_qat_zstd\x18\x06 \x01(\x08\x12N\n\x1bqat_zstd_fallback_threshold\x18\x07 \x01(\x0b\x32\x1c.google.protobuf.UInt32ValueB\x0b\xfa\x42\x08*\x06\x18\x80\x80\x04(\x00\"\x80\x01\n\x08Strategy\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\x08\n\x04\x46\x41ST\x10\x01\x12\t\n\x05\x44\x46\x41ST\x10\x02\x12\n\n\x06GREEDY\x10\x03\x12\x08\n\x04LAZY\x10\x04\x12\t\n\x05LAZY2\x10\x05\x12\x0b\n\x07\x42TLAZY2\x10\x06\x12\t\n\x05\x42TOPT\x10\x07\x12\x0b\n\x07\x42TULTRA\x10\x08\x12\x0c\n\x08\x42TULTRA2\x10\tB\xc7\x01\nEio.envoyproxy.envoy.extensions.compression.qatzstd.compressor.v3alphaB\x0cQatzstdProtoP\x01Zfgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/compression/qatzstd/compressor/v3alpha\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'contrib.envoy.extensions.compression.qatzstd.compressor.v3alpha.qatzstd_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nEio.envoyproxy.envoy.extensions.compression.qatzstd.compressor.v3alphaB\014QatzstdProtoP\001Zfgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/compression/qatzstd/compressor/v3alpha\272\200\310\321\006\002\020\002'
  _globals['_QATZSTD'].fields_by_name['compression_level']._loaded_options = None
  _globals['_QATZSTD'].fields_by_name['compression_level']._serialized_options = b'\372B\006*\004\030\026(\001'
  _globals['_QATZSTD'].fields_by_name['strategy']._loaded_options = None
  _globals['_QATZSTD'].fields_by_name['strategy']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_QATZSTD'].fields_by_name['chunk_size']._loaded_options = None
  _globals['_QATZSTD'].fields_by_name['chunk_size']._serialized_options = b'\372B\t*\007\030\200\200\004(\200 '
  _globals['_QATZSTD'].fields_by_name['qat_zstd_fallback_threshold']._loaded_options = None
  _globals['_QATZSTD'].fields_by_name['qat_zstd_fallback_threshold']._serialized_options = b'\372B\010*\006\030\200\200\004(\000'
  _globals['_QATZSTD']._serialized_start=227
  _globals['_QATZSTD']._serialized_end=732
  _globals['_QATZSTD_STRATEGY']._serialized_start=604
  _globals['_QATZSTD_STRATEGY']._serialized_end=732
# @@protoc_insertion_point(module_scope)



# Copyright 2019 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import asyncio
import logging
import os
import sys

import grpc
from grpc.experimental import aio

from tests.interop import client as interop_client_lib
from tests_aio.interop import methods

_LOGGER = logging.getLogger(__name__)
_LOGGER.setLevel(logging.DEBUG)


def _create_channel(args):
    target = f"{args.server_host}:{args.server_port}"

    if (
        args.use_tls
        or args.use_alts
        or args.custom_credentials_type is not None
    ):
        (
            channel_credentials,
            options,
        ) = interop_client_lib.get_secure_channel_parameters(args)
        return aio.secure_channel(target, channel_credentials, options)
    else:
        return aio.insecure_channel(target)


def _test_case_from_arg(test_case_arg):
    for test_case in methods.TestCase:
        if test_case_arg == test_case.value:
            return test_case
    else:
        raise ValueError('No test case "%s"!' % test_case_arg)


async def test_interoperability():
    args = interop_client_lib.parse_interop_client_args(sys.argv)
    channel = _create_channel(args)
    stub = interop_client_lib.create_stub(channel, args)
    test_case = _test_case_from_arg(args.test_case)
    await methods.test_interoperability(test_case, stub, args)


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    asyncio.get_event_loop().set_debug(True)
    asyncio.get_event_loop().run_until_complete(test_interoperability())



# Copyright 2026 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The Python AsyncIO example of the GRPC helloworld.Greeter server with observability enabled."""

import asyncio
import collections
import faulthandler
import logging

import grpc
import grpc_observability
import helloworld_pb2
import helloworld_pb2_grpc
import open_telemetry_exporter
from opentelemetry.sdk import metrics as otel_metrics
from opentelemetry.sdk.metrics import export as otel_metrics_export

_OTEL_EXPORT_INTERVAL_S = 0.5
_SERVER_PORT = "50051"


class Greeter(helloworld_pb2_grpc.GreeterServicer):
    async def SayHello(self, request, context):
        message = request.name
        return helloworld_pb2.HelloReply(message=f"Hello {message}")


async def serve():
    all_metrics = collections.defaultdict(list)
    otel_exporter = open_telemetry_exporter.OTelMetricExporter(
        all_metrics, print_live=False
    )
    reader = otel_metrics_export.PeriodicExportingMetricReader(
        exporter=otel_exporter,
        export_interval_millis=_OTEL_EXPORT_INTERVAL_S * 1000,
    )
    provider = otel_metrics.MeterProvider(metric_readers=(reader,))

    otel_plugin = grpc_observability.OpenTelemetryPlugin(
        meter_provider=provider
    )
    otel_plugin.register_global()

    server = grpc.aio.server()
    helloworld_pb2_grpc.add_GreeterServicer_to_server(Greeter(), server)
    server.add_insecure_port("[::]:" + _SERVER_PORT)
    await server.start()
    print("Server started, listening on " + _SERVER_PORT)

    # Wait for the client to send an RPC, and for metrics to export.
    await asyncio.sleep(10)
    print("Metrics exported on Server side:")
    for metric in all_metrics:
        print(metric)

    await server.stop(0)
    otel_plugin.deregister_global()
    provider.shutdown()


if __name__ == "__main__":
    faulthandler.enable()
    logging.basicConfig()
    asyncio.run(serve())



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/type/v3/semantic_version.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/type/v3/semantic_version.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$envoy/type/v3/semantic_version.proto\x12\renvoy.type.v3\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\"o\n\x0fSemanticVersion\x12\x14\n\x0cmajor_number\x18\x01 \x01(\r\x12\x14\n\x0cminor_number\x18\x02 \x01(\r\x12\r\n\x05patch\x18\x03 \x01(\r:!\x9a\xc5\x88\x1e\x1c\n\x1a\x65nvoy.type.SemanticVersionBz\n\x1bio.envoyproxy.envoy.type.v3B\x14SemanticVersionProtoP\x01Z;github.com/envoyproxy/go-control-plane/envoy/type/v3;typev3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.type.v3.semantic_version_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\033io.envoyproxy.envoy.type.v3B\024SemanticVersionProtoP\001Z;github.com/envoyproxy/go-control-plane/envoy/type/v3;typev3\272\200\310\321\006\002\020\002'
  _globals['_SEMANTICVERSION']._loaded_options = None
  _globals['_SEMANTICVERSION']._serialized_options = b'\232\305\210\036\034\n\032envoy.type.SemanticVersion'
  _globals['_SEMANTICVERSION']._serialized_start=121
  _globals['_SEMANTICVERSION']._serialized_end=232
# @@protoc_insertion_point(module_scope)



# Copyright 2019 The gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test of dynamic stub import API."""

import contextlib
import functools
import logging
import multiprocessing
import os
import sys
import unittest

from tests.unit import test_common

_DATA_DIR = os.path.join("tests", "unit", "data")


@contextlib.contextmanager
def _grpc_tools_unimportable():
    original_sys_path = sys.path
    sys.path = [path for path in sys.path if "grpcio_tools" not in str(path)]
    try:
        import grpc_tools
    except ImportError:
        pass
    else:
        del grpc_tools
        sys.path = original_sys_path
        raise unittest.SkipTest("Failed to make grpc_tools unimportable.")
    try:
        yield
    finally:
        sys.path = original_sys_path


def _collect_errors(fn):
    @functools.wraps(fn)
    def _wrapped(error_queue):
        try:
            fn()
        except Exception as e:
            error_queue.put(e)
            raise

    return _wrapped


def _run_in_subprocess(test_case):
    # The default start method of multiprocessing in linux has changed to
    # `forkserver` instead of `fork` from Python 3.14. This causes problems
    # with pickling target methods that use decorators.
    # Hence manually set to use the `fork` start method.
    if sys.version_info >= (3, 14) and ("linux" in sys.platform):
        multiprocessing.set_start_method("fork", force=True)

    sys.path.insert(
        0, os.path.join(os.path.realpath(os.path.dirname(__file__)), "..")
    )
    error_queue = multiprocessing.Queue()
    proc = multiprocessing.Process(target=test_case, args=(error_queue,))
    proc.start()
    proc.join()
    sys.path.pop(0)
    if not error_queue.empty():
        raise error_queue.get()
    assert proc.exitcode == 0, "Process exited with code {}".format(
        proc.exitcode
    )


def _assert_unimplemented(msg_substr):
    import grpc

    try:
        protos, services = grpc.protos_and_services(
            "tests/unit/data/foo/bar.proto"
        )
    except NotImplementedError as e:
        assert msg_substr in str(e), "{} was not in '{}'".format(
            msg_substr, str(e)
        )
    else:
        assert False, "Did not raise NotImplementedError"


@_collect_errors
def _test_sunny_day():
    import grpc

    protos, services = grpc.protos_and_services(
        os.path.join(_DATA_DIR, "foo", "bar.proto")
    )
    assert protos.BarMessage is not None
    assert services.BarStub is not None


@_collect_errors
def _test_well_known_types():
    import grpc

    protos, services = grpc.protos_and_services(
        os.path.join(_DATA_DIR, "foo", "bar_with_wkt.proto")
    )
    assert protos.BarMessage is not None
    assert services.BarStub is not None


@_collect_errors
def _test_grpc_tools_unimportable():
    with _grpc_tools_unimportable():
        _assert_unimplemented("grpcio-tools")


# NOTE(rbellevi): multiprocessing.Process fails to pickle function objects
# when they do not come from the "__main__" module, so this test passes
# if run directly on Windows or MacOS, but not if started by the test runner.
@unittest.skipIf(
    os.name == "nt" or "darwin" in sys.platform,
    "Windows and MacOS multiprocessing unsupported",
)
class DynamicStubTest(unittest.TestCase):
    def test_sunny_day(self):
        _run_in_subprocess(_test_sunny_day)

    def test_well_known_types(self):
        _run_in_subprocess(_test_well_known_types)

    def test_grpc_tools_unimportable(self):
        _run_in_subprocess(_test_grpc_tools_unimportable)


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)



# Copyright 2023 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import threading
import time
from typing import Any, AnyStr, Dict, Iterable, List, Optional, Set, Union

import grpc
from grpc_observability import _cyobservability
from grpc_observability import _observability
from grpc_observability import _open_telemetry_measures
from grpc_observability._cyobservability import MetricsName
from grpc_observability._cyobservability import PLUGIN_IDENTIFIER_SEP
from grpc_observability._observability import OptionalLabelType
from grpc_observability._observability import StatsData
from opentelemetry.metrics import Counter
from opentelemetry.metrics import Histogram
from opentelemetry.metrics import Meter

_LOGGER = logging.getLogger(__name__)

ClientCallTracerCapsule = Any  # it appears only once in the function signature
ServerCallTracerFactoryCapsule = (
    Any  # it appears only once in the function signature
)
grpc_observability = Any  # grpc_observability.py imports this module.
OpenTelemetryPlugin = Any  # _open_telemetry_plugin.py imports this module.
OpenTelemetryPluginOption = (
    Any  # _open_telemetry_plugin.py imports this module.
)

GRPC_METHOD_LABEL = "grpc.method"
GRPC_TARGET_LABEL = "grpc.target"
GRPC_CLIENT_METRIC_PREFIX = "grpc.client"
GRPC_OTHER_LABEL_VALUE = "other"
_observability_lock: threading.RLock = threading.RLock()
_OPEN_TELEMETRY_OBSERVABILITY: Optional["OpenTelemetryObservability"] = None

GRPC_STATUS_CODE_TO_STRING = {
    grpc.StatusCode.OK: "OK",
    grpc.StatusCode.CANCELLED: "CANCELLED",
    grpc.StatusCode.UNKNOWN: "UNKNOWN",
    grpc.StatusCode.INVALID_ARGUMENT: "INVALID_ARGUMENT",
    grpc.StatusCode.DEADLINE_EXCEEDED: "DEADLINE_EXCEEDED",
    grpc.StatusCode.NOT_FOUND: "NOT_FOUND",
    grpc.StatusCode.ALREADY_EXISTS: "ALREADY_EXISTS",
    grpc.StatusCode.PERMISSION_DENIED: "PERMISSION_DENIED",
    grpc.StatusCode.UNAUTHENTICATED: "UNAUTHENTICATED",
    grpc.StatusCode.RESOURCE_EXHAUSTED: "RESOURCE_EXHAUSTED",
    grpc.StatusCode.FAILED_PRECONDITION: "FAILED_PRECONDITION",
    grpc.StatusCode.ABORTED: "ABORTED",
    grpc.StatusCode.OUT_OF_RANGE: "OUT_OF_RANGE",
    grpc.StatusCode.UNIMPLEMENTED: "UNIMPLEMENTED",
    grpc.StatusCode.INTERNAL: "INTERNAL",
    grpc.StatusCode.UNAVAILABLE: "UNAVAILABLE",
    grpc.StatusCode.DATA_LOSS: "DATA_LOSS",
}


class _OpenTelemetryPlugin:
    _plugin: OpenTelemetryPlugin
    _metric_to_recorder: Dict[MetricsName, Union[Counter, Histogram]]
    _enabled_client_plugin_options: Optional[List[OpenTelemetryPluginOption]]
    _enabled_server_plugin_options: Optional[List[OpenTelemetryPluginOption]]
    identifier: str

    def __init__(self, plugin: OpenTelemetryPlugin):
        self._plugin = plugin
        self._metric_to_recorder = {}
        self.identifier = str(id(self))
        self._enabled_client_plugin_options = None
        self._enabled_server_plugin_options = None

        meter_provider = self._plugin.meter_provider
        if meter_provider:
            meter = meter_provider.get_meter("grpc-python", grpc.__version__)
            enabled_metrics = _open_telemetry_measures.base_metrics()
            self._metric_to_recorder = self._register_metrics(
                meter, enabled_metrics
            )

    def _should_record(self, stats_data: StatsData) -> bool:
        # Decide if this plugin should record the stats_data.
        return stats_data.name in self._metric_to_recorder

    def _record_stats_data(self, stats_data: StatsData) -> None:
        recorder = self._metric_to_recorder[stats_data.name]
        enabled_plugin_options = []
        if GRPC_CLIENT_METRIC_PREFIX in recorder.name:
            enabled_plugin_options = self._enabled_client_plugin_options
        else:
            enabled_plugin_options = self._enabled_server_plugin_options
        # Only deserialize labels if we need add exchanged labels.
        if stats_data.include_exchange_labels:
            deserialized_labels = self._deserialize_labels(
                stats_data.labels, enabled_plugin_options
            )
        else:
            deserialized_labels = stats_data.labels
        labels = self._maybe_add_labels(
            stats_data.include_exchange_labels,
            deserialized_labels,
            enabled_plugin_options,
        )
        decoded_labels = self.decode_labels(labels)

        target = decoded_labels.get(GRPC_TARGET_LABEL, "")
        if not self._plugin.target_attribute_filter(target):
            # Filter target name.
            decoded_labels[GRPC_TARGET_LABEL] = GRPC_OTHER_LABEL_VALUE

        method = decoded_labels.get(GRPC_METHOD_LABEL, "")
        if not (
            stats_data.registered_method
            or self._plugin.generic_method_attribute_filter(method)
        ):
            # Filter method name if it's not registered method and
            # generic_method_attribute_filter returns false.
            decoded_labels[GRPC_METHOD_LABEL] = GRPC_OTHER_LABEL_VALUE

        value = 0
        if stats_data.measure_double:
            value = stats_data.value_float
        else:
            value = stats_data.value_int
        if isinstance(recorder, Counter):
            recorder.add(value, attributes=decoded_labels)
        elif isinstance(recorder, Histogram):
            recorder.record(value, attributes=decoded_labels)

    def maybe_record_stats_data(self, stats_data: StatsData) -> None:
        # Records stats data to MeterProvider.
        if self._should_record(stats_data):
            self._record_stats_data(stats_data)

    def get_client_exchange_labels(self) -> Dict[str, AnyStr]:
        """Get labels used for client side Metadata Exchange."""
        labels_for_exchange = {}
        for plugin_option in self._enabled_client_plugin_options:
            if hasattr(plugin_option, "get_label_injector") and hasattr(
                plugin_option.get_label_injector(), "get_labels_for_exchange"
            ):
                labels_for_exchange.update(
                    plugin_option.get_label_injector().get_labels_for_exchange()
                )
        return labels_for_exchange

    def get_server_exchange_labels(self) -> Dict[str, str]:
        """Get labels used for server side Metadata Exchange."""
        labels_for_exchange = {}
        for plugin_option in self._enabled_server_plugin_options:
            if hasattr(plugin_option, "get_label_injector") and hasattr(
                plugin_option.get_label_injector(), "get_labels_for_exchange"
            ):
                labels_for_exchange.update(
                    plugin_option.get_label_injector().get_labels_for_exchange()
                )
        return labels_for_exchange

    def activate_client_plugin_options(self, target: bytes) -> None:
        """Activate client plugin options based on option settings."""
        target_str = target.decode("utf-8", "replace")
        if not self._enabled_client_plugin_options:
            self._enabled_client_plugin_options = []
            for plugin_option in self._plugin.plugin_options:
                if hasattr(
                    plugin_option, "is_active_on_client_channel"
                ) and plugin_option.is_active_on_client_channel(target_str):
                    self._enabled_client_plugin_options.append(plugin_option)

    def activate_server_plugin_options(self, xds: bool) -> None:
        """Activate server plugin options based on option settings."""
        if not self._enabled_server_plugin_options:
            self._enabled_server_plugin_options = []
            for plugin_option in self._plugin.plugin_options:
                if hasattr(
                    plugin_option, "is_active_on_server"
                ) and plugin_option.is_active_on_server(xds):
                    self._enabled_server_plugin_options.append(plugin_option)

    @staticmethod
    def _deserialize_labels(
        labels: Dict[str, AnyStr],
        enabled_plugin_options: List[OpenTelemetryPluginOption],
    ) -> Dict[str, AnyStr]:
        for plugin_option in enabled_plugin_options:
            if all(
                [
                    hasattr(plugin_option, "get_label_injector"),
                    hasattr(
                        plugin_option.get_label_injector(), "deserialize_labels"
                    ),
                ]
            ):
                labels = plugin_option.get_label_injector().deserialize_labels(
                    labels
                )
        return labels

    @staticmethod
    def _maybe_add_labels(
        include_exchange_labels: bool,
        labels: Dict[str, str],
        enabled_plugin_options: List[OpenTelemetryPluginOption],
    ) -> Dict[str, AnyStr]:
        for plugin_option in enabled_plugin_options:
            if all(
                [
                    hasattr(plugin_option, "get_label_injector"),
                    hasattr(
                        plugin_option.get_label_injector(),
                        "get_additional_labels",
                    ),
                ]
            ):
                labels.update(
                    plugin_option.get_label_injector().get_additional_labels(
                        include_exchange_labels
                    )
                )
        return labels

    def get_enabled_optional_labels(self) -> List[OptionalLabelType]:
        return self._plugin._get_enabled_optional_labels()

    @staticmethod
    def _register_metrics(
        meter: Meter, metrics: List[_open_telemetry_measures.Metric]
    ) -> Dict[MetricsName, Union[Counter, Histogram]]:
        metric_to_recorder_map = {}
        recorder = None
        for metric in metrics:
            if metric == _open_telemetry_measures.CLIENT_ATTEMPT_STARTED:
                recorder = meter.create_counter(
                    name=metric.name,
                    unit=metric.unit,
                    description=metric.description,
                )
            elif metric in (
                _open_telemetry_measures.CLIENT_ATTEMPT_DURATION,
                _open_telemetry_measures.CLIENT_RPC_DURATION,
                _open_telemetry_measures.CLIENT_ATTEMPT_SEND_BYTES,
                _open_telemetry_measures.CLIENT_ATTEMPT_RECEIVED_BYTES,
            ):
                recorder = meter.create_histogram(
                    name=metric.name,
                    unit=metric.unit,
                    description=metric.description,
                )
            elif metric == _open_telemetry_measures.SERVER_STARTED_RPCS:
                recorder = meter.create_counter(
                    name=metric.name,
                    unit=metric.unit,
                    description=metric.description,
                )
            elif metric in (
                _open_telemetry_measures.SERVER_RPC_DURATION,
                _open_telemetry_measures.SERVER_RPC_SEND_BYTES,
                _open_telemetry_measures.SERVER_RPC_RECEIVED_BYTES,
            ):
                recorder = meter.create_histogram(
                    name=metric.name,
                    unit=metric.unit,
                    description=metric.description,
                )
            metric_to_recorder_map[metric.cyname] = recorder
        return metric_to_recorder_map

    @staticmethod
    def decode_labels(labels: Dict[str, AnyStr]) -> Dict[str, str]:
        decoded_labels = {}
        for key, value in labels.items():
            if isinstance(value, bytes):
                value = value.decode()
            decoded_labels[key] = value
        return decoded_labels


def start_open_telemetry_observability(
    *,
    plugins: Iterable[_OpenTelemetryPlugin],
) -> None:
    _start_open_telemetry_observability(
        OpenTelemetryObservability(plugins=plugins)
    )


def end_open_telemetry_observability() -> None:
    _end_open_telemetry_observability()


class _OpenTelemetryExporterDelegator(_observability.Exporter):
    _plugins: Iterable[_OpenTelemetryPlugin]

    def __init__(self, plugins: Iterable[_OpenTelemetryPlugin]):
        self._plugins = plugins

    def export_stats_data(
        self, stats_data: List[_observability.StatsData]
    ) -> None:
        # Records stats data to MeterProvider.
        for data in stats_data:
            for plugin in self._plugins:
                plugin.maybe_record_stats_data(data)

    def export_tracing_data(
        self, tracing_data: List[_observability.TracingData]
    ) -> None:
        pass


# pylint: disable=no-self-use
class OpenTelemetryObservability(grpc._observability.ObservabilityPlugin):
    """OpenTelemetry based plugin implementation.

    This is class is part of an EXPERIMENTAL API.

    Args:
      plugins: _OpenTelemetryPlugins to enable.
    """

    _exporter: "grpc_observability.Exporter"
    _plugins: List[_OpenTelemetryPlugin]
    _registered_methods: Set[bytes]
    _client_option_activated: bool
    _server_option_activated: bool

    def __init__(
        self,
        *,
        plugins: Iterable[_OpenTelemetryPlugin],
    ):
        self._exporter = _OpenTelemetryExporterDelegator(plugins)
        self._registered_methods = set()
        self._plugins = list(plugins)
        self._client_option_activated = False
        self._server_option_activated = False

    def observability_init(self):
        try:
            _cyobservability.activate_stats()
            self.set_stats(True)
        except Exception as e:  # pylint: disable=broad-except
            error_msg = f"Activate observability metrics failed with: {e}"
            raise ValueError(error_msg)

        try:
            _cyobservability.cyobservability_init(self._exporter)
        # TODO(xuanwn): Use specific exceptions
        except Exception as e:  # pylint: disable=broad-except
            _LOGGER.exception("Initiate observability failed with: %s", e)

        grpc._observability.observability_init(self)

    def observability_deinit(self) -> None:
        # Sleep so we don't loss any data. If we shutdown export thread
        # immediately after exit, it's possible that core didn't call RecordEnd
        # in callTracer, and all data recorded by calling RecordEnd will be
        # lost.
        # CENSUS_EXPORT_BATCH_INTERVAL_SECS: The time equals to the time in
        # AwaitNextBatchLocked.
        # TODO(xuanwn): explicit synchronization
        # https://github.com/grpc/grpc/issues/33262
        time.sleep(_cyobservability.CENSUS_EXPORT_BATCH_INTERVAL_SECS)
        self.set_tracing(False)
        self.set_stats(False)
        _cyobservability.observability_deinit()
        grpc._observability.observability_deinit()

    def create_client_call_tracer(
        self, method_name: bytes, target: bytes
    ) -> ClientCallTracerCapsule:
        trace_id = b"TRACE_ID"
        self._maybe_activate_client_plugin_options(target)
        exchange_labels = self._get_client_exchange_labels()
        enabled_optional_labels = set()
        for plugin in self._plugins:
            enabled_optional_labels.update(plugin.get_enabled_optional_labels())

        capsule = _cyobservability.create_client_call_tracer(
            method_name,
            target,
            trace_id,
            self._get_identifier(),
            exchange_labels,
            enabled_optional_labels,
            method_name in self._registered_methods,
        )
        return capsule

    def create_server_call_tracer_factory(
        self,
        *,
        xds: bool = False,
    ) -> Optional[ServerCallTracerFactoryCapsule]:
        capsule = None
        self._maybe_activate_server_plugin_options(xds)
        exchange_labels = self._get_server_exchange_labels()
        capsule = _cyobservability.create_server_call_tracer_factory_capsule(
            exchange_labels, self._get_identifier()
        )
        return capsule

    def save_trace_context(
        self, trace_id: str, span_id: str, is_sampled: bool
    ) -> None:
        pass

    def record_rpc_latency(
        self,
        method: str,
        target: str,
        rpc_latency: float,
        status_code: grpc.StatusCode,
    ) -> None:
        status_code = GRPC_STATUS_CODE_TO_STRING.get(status_code, "UNKNOWN")
        encoded_method = method.encode("utf8")
        _cyobservability._record_rpc_latency(
            self._exporter,
            method,
            target,
            rpc_latency,
            status_code,
            self._get_identifier(),
            encoded_method in self._registered_methods,
        )

    def save_registered_method(self, method_name: bytes) -> None:
        self._registered_methods.add(method_name)

    def _get_client_exchange_labels(self) -> Dict[str, AnyStr]:
        client_exchange_labels = {}
        for _plugin in self._plugins:
            client_exchange_labels.update(_plugin.get_client_exchange_labels())
        return client_exchange_labels

    def _get_server_exchange_labels(self) -> Dict[str, AnyStr]:
        server_exchange_labels = {}
        for _plugin in self._plugins:
            server_exchange_labels.update(_plugin.get_server_exchange_labels())
        return server_exchange_labels

    def _maybe_activate_client_plugin_options(self, target: bytes) -> None:
        if not self._client_option_activated:
            for _plugin in self._plugins:
                _plugin.activate_client_plugin_options(target)
            self._client_option_activated = True

    def _maybe_activate_server_plugin_options(self, xds: bool) -> None:
        if not self._server_option_activated:
            for _plugin in self._plugins:
                _plugin.activate_server_plugin_options(xds)
            self._server_option_activated = True

    def _get_identifier(self) -> str:
        return PLUGIN_IDENTIFIER_SEP.join(
            _plugin.identifier for _plugin in self._plugins
        )

    def get_enabled_optional_labels(self) -> List[OptionalLabelType]:
        return []


def _start_open_telemetry_observability(
    otel_o11y: OpenTelemetryObservability,
) -> None:
    global _OPEN_TELEMETRY_OBSERVABILITY  # pylint: disable=global-statement # noqa: PLW0603
    with _observability_lock:
        if _OPEN_TELEMETRY_OBSERVABILITY is None:
            _OPEN_TELEMETRY_OBSERVABILITY = otel_o11y
            _OPEN_TELEMETRY_OBSERVABILITY.observability_init()
        else:
            error_msg = "gPRC Python observability was already initialized!"
            raise RuntimeError(error_msg)


def _end_open_telemetry_observability() -> None:
    global _OPEN_TELEMETRY_OBSERVABILITY  # pylint: disable=global-statement # noqa: PLW0603
    with _observability_lock:
        if not _OPEN_TELEMETRY_OBSERVABILITY:
            error_msg = "Trying to end gPRC Python observability without initialize first!"
            raise RuntimeError(error_msg)
        _OPEN_TELEMETRY_OBSERVABILITY.observability_deinit()
        _OPEN_TELEMETRY_OBSERVABILITY = None



# Copyright 2018 The gRPC Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from concurrent import futures
import logging
import unittest

import grpc

from tests.unit import resources
from tests.unit import test_common
from tests.unit.framework.common import test_constants

_REQUEST = b""
_RESPONSE = b"response"
_REGISTERED_RESPONSE = b"registered_response"

_SERVICE_NAME = "test"
_UNARY_UNARY = "UnaryUnary"
_UNARY_UNARY_REGISTERED = "UnaryUnaryRegistered"
_UNARY_STREAM = "UnaryStream"
_STREAM_UNARY = "StreamUnary"
_STREAM_STREAM = "StreamStream"


class _ActualGenericRpcHandler(grpc.GenericRpcHandler):
    def service(self, handler_call_details):
        return None


def handle_unary_unary(request, servicer_context):
    return _RESPONSE


def handle_registered_unary_unary(request, servicer_context):
    return _REGISTERED_RESPONSE


def handle_unary_stream(request, servicer_context):
    for _ in range(test_constants.STREAM_LENGTH):
        yield _RESPONSE


def handle_stream_unary(request_iterator, servicer_context):
    for request in request_iterator:
        pass
    return _RESPONSE


def handle_stream_stream(request_iterator, servicer_context):
    for request in request_iterator:
        yield _RESPONSE


class _MethodHandler(grpc.RpcMethodHandler):
    def __init__(self, request_streaming, response_streaming, registered=False):
        self.request_streaming = request_streaming
        self.response_streaming = response_streaming
        self.request_deserializer = None
        self.response_serializer = None
        self.unary_unary = None
        self.unary_stream = None
        self.stream_unary = None
        self.stream_stream = None
        if self.request_streaming and self.response_streaming:
            self.stream_stream = handle_stream_stream
        elif self.request_streaming:
            self.stream_unary = handle_stream_unary
        elif self.response_streaming:
            self.unary_stream = handle_unary_stream
        else:
            if registered:
                self.unary_unary = handle_registered_unary_unary
            else:
                self.unary_unary = handle_unary_unary


class _GenericHandler(grpc.GenericRpcHandler):
    def service(self, handler_call_details):
        if handler_call_details.method == _UNARY_UNARY:
            return _MethodHandler(False, False)
        elif handler_call_details.method == _UNARY_STREAM:
            return _MethodHandler(False, True)
        elif handler_call_details.method == _STREAM_UNARY:
            return _MethodHandler(True, False)
        elif handler_call_details.method == _STREAM_STREAM:
            return _MethodHandler(True, True)
        elif handler_call_details.method == grpc._common.fully_qualified_method(
            _SERVICE_NAME, _UNARY_UNARY_REGISTERED
        ):
            return _MethodHandler(False, False)
        else:
            return None


class _GenericHandlerWithRegisteredName(grpc.GenericRpcHandler):
    def service(self, handler_call_details):
        if handler_call_details.method == grpc._common.fully_qualified_method(
            _SERVICE_NAME, _UNARY_UNARY_REGISTERED
        ):
            return _MethodHandler(False, False)
        else:
            return None


_REGISTERED_METHOD_HANDLERS = {
    _UNARY_UNARY_REGISTERED: _MethodHandler(False, False, True),
}


class ServerTest(unittest.TestCase):
    def test_not_a_generic_rpc_handler_at_construction(self):
        with self.assertRaises(AttributeError) as exception_context:
            grpc.server(
                futures.ThreadPoolExecutor(max_workers=5),
                handlers=[
                    _ActualGenericRpcHandler(),
                    object(),
                ],
            )
        self.assertIn(
            "grpc.GenericRpcHandler", str(exception_context.exception)
        )

    def test_not_a_generic_rpc_handler_after_construction(self):
        server = grpc.server(futures.ThreadPoolExecutor(max_workers=5))
        with self.assertRaises(AttributeError) as exception_context:
            server.add_generic_rpc_handlers(
                [
                    _ActualGenericRpcHandler(),
                    object(),
                ]
            )
        self.assertIn(
            "grpc.GenericRpcHandler", str(exception_context.exception)
        )

    def test_failed_port_binding_exception(self):
        server = grpc.server(None, options=(("grpc.so_reuseport", 0),))
        port = server.add_insecure_port("localhost:0")
        bind_address = "localhost:%d" % port

        with self.assertRaises(RuntimeError):
            server.add_insecure_port(bind_address)

        server_credentials = grpc.ssl_server_credentials(
            [(resources.private_key(), resources.certificate_chain())]
        )
        with self.assertRaises(RuntimeError):
            server.add_secure_port(bind_address, server_credentials)


class ServerHandlerTest(unittest.TestCase):
    def tearDown(self):
        self._server.stop(0)
        self._channel.close()

    def test_generic_unary_unary_handler(self):
        self._server = test_common.test_server()
        port = self._server.add_insecure_port("[::]:0")
        self._server.start()
        self._server.add_generic_rpc_handlers((_GenericHandler(),))
        self._channel = grpc.insecure_channel("localhost:%d" % port)

        response = self._channel.unary_unary(
            _UNARY_UNARY,
            _registered_method=True,
        )(_REQUEST)
        self.assertEqual(_RESPONSE, response)

    def test_generic_unary_stream_handler(self):
        self._server = test_common.test_server()
        self._server.add_generic_rpc_handlers((_GenericHandler(),))
        port = self._server.add_insecure_port("[::]:0")
        self._server.start()
        self._channel = grpc.insecure_channel("localhost:%d" % port)

        response_iterator = self._channel.unary_stream(
            _UNARY_STREAM,
            _registered_method=True,
        )(_REQUEST)
        self.assertSequenceEqual(
            [_RESPONSE] * test_constants.STREAM_LENGTH, list(response_iterator)
        )

    def test_generic_stream_unary_handler(self):
        self._server = test_common.test_server()
        self._server.add_generic_rpc_handlers((_GenericHandler(),))
        port = self._server.add_insecure_port("[::]:0")
        self._server.start()
        self._channel = grpc.insecure_channel("localhost:%d" % port)

        response = self._channel.stream_unary(
            _STREAM_UNARY,
            _registered_method=True,
        )(iter([_REQUEST] * test_constants.STREAM_LENGTH))
        self.assertEqual(_RESPONSE, response)

    def test_generic_stream_stream_handler(self):
        self._server = test_common.test_server()
        self._server.add_generic_rpc_handlers((_GenericHandler(),))
        port = self._server.add_insecure_port("[::]:0")
        self._server.start()
        self._channel = grpc.insecure_channel("localhost:%d" % port)

        response_iterator = self._channel.stream_stream(
            _STREAM_STREAM,
            _registered_method=True,
        )(iter([_REQUEST] * test_constants.STREAM_LENGTH))
        self.assertSequenceEqual(
            [_RESPONSE] * test_constants.STREAM_LENGTH, list(response_iterator)
        )

    def test_add_generic_handler_after_server_start(self):
        self._server = test_common.test_server()
        port = self._server.add_insecure_port("[::]:0")
        self._server.start()
        self._server.add_generic_rpc_handlers((_GenericHandler(),))
        self._channel = grpc.insecure_channel("localhost:%d" % port)

        response = self._channel.unary_unary(
            _UNARY_UNARY,
            _registered_method=True,
        )(_REQUEST)
        self.assertEqual(_RESPONSE, response)

    def test_add_registered_handler_after_server_start(self):
        self._server = test_common.test_server()
        port = self._server.add_insecure_port("[::]:0")
        self._server.start()
        self._server.add_registered_method_handlers(
            _SERVICE_NAME, _REGISTERED_METHOD_HANDLERS
        )
        self._channel = grpc.insecure_channel("localhost:%d" % port)

        with self.assertRaises(grpc.RpcError) as exception_context:
            self._channel.unary_unary(
                grpc._common.fully_qualified_method(
                    _SERVICE_NAME, _UNARY_UNARY_REGISTERED
                ),
                _registered_method=True,
            )(_REQUEST)

        self.assertIn("Method not found", str(exception_context.exception))

    def test_server_with_both_registered_and_generic_handlers(self):
        self._server = test_common.test_server()
        self._server.add_generic_rpc_handlers((_GenericHandler(),))
        self._server.add_registered_method_handlers(
            _SERVICE_NAME, _REGISTERED_METHOD_HANDLERS
        )
        port = self._server.add_insecure_port("[::]:0")
        self._server.start()
        self._channel = grpc.insecure_channel("localhost:%d" % port)

        generic_response = self._channel.unary_unary(
            _UNARY_UNARY,
            _registered_method=True,
        )(_REQUEST)
        self.assertEqual(_RESPONSE, generic_response)

        registered_response = self._channel.unary_unary(
            grpc._common.fully_qualified_method(
                _SERVICE_NAME, _UNARY_UNARY_REGISTERED
            ),
            _registered_method=True,
        )(_REQUEST)
        self.assertEqual(_REGISTERED_RESPONSE, registered_response)

    def test_server_registered_handler_take_precedence(self):
        # Test if the same method have both generic and registered handler,
        # registered handler will take precedence.
        self._server = test_common.test_server()
        self._server.add_generic_rpc_handlers(
            (_GenericHandlerWithRegisteredName(),)
        )
        self._server.add_registered_method_handlers(
            _SERVICE_NAME, _REGISTERED_METHOD_HANDLERS
        )
        port = self._server.add_insecure_port("[::]:0")
        self._server.start()
        self._channel = grpc.insecure_channel("localhost:%d" % port)

        registered_response = self._channel.unary_unary(
            grpc._common.fully_qualified_method(
                _SERVICE_NAME, _UNARY_UNARY_REGISTERED
            ),
            _registered_method=True,
        )(_REQUEST)
        self.assertEqual(_REGISTERED_RESPONSE, registered_response)


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: contrib/envoy/extensions/filters/network/generic_proxy/codecs/dubbo/v3/dubbo.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nRcontrib/envoy/extensions/filters/network/generic_proxy/codecs/dubbo/v3/dubbo.proto\x12>envoy.extensions.filters.network.generic_proxy.codecs.dubbo.v3\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\"\x12\n\x10\x44ubboCodecConfigB\xe3\x01\nLio.envoyproxy.envoy.extensions.filters.network.generic_proxy.codecs.dubbo.v3B\nDubboProtoP\x01Zugithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/generic_proxy/codecs/dubbo/v3;dubbov3\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'contrib.envoy.extensions.filters.network.generic_proxy.codecs.dubbo.v3.dubbo_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\nLio.envoyproxy.envoy.extensions.filters.network.generic_proxy.codecs.dubbo.v3B\nDubboProtoP\001Zugithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/generic_proxy/codecs/dubbo/v3;dubbov3\272\200\310\321\006\002\020\002\322\306\244\341\006\002\010\001'
  _globals['_DUBBOCODECCONFIG']._serialized_start=214
  _globals['_DUBBOCODECCONFIG']._serialized_end=232
# @@protoc_insertion_point(module_scope)



# Copyright 2020 The gRPC Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests the behaviour of the Call classes under a secure channel."""

import logging
import unittest

import grpc
from grpc.experimental import aio

from src.proto.grpc.testing import messages_pb2
from src.proto.grpc.testing import test_pb2_grpc
from tests.unit import resources
from tests_aio.unit._test_base import AioTestBase
from tests_aio.unit._test_server import start_test_server

_SERVER_HOST_OVERRIDE = "foo.test.google.fr"
_NUM_STREAM_RESPONSES = 5
_RESPONSE_PAYLOAD_SIZE = 42


class _SecureCallMixin:
    """A Mixin to run the call tests over a secure channel."""

    async def setUp(self):
        server_credentials = grpc.ssl_server_credentials(
            [(resources.private_key(), resources.certificate_chain())]
        )
        channel_credentials = grpc.ssl_channel_credentials(
            resources.test_root_certificates()
        )

        self._server_address, self._server = await start_test_server(
            secure=True, server_credentials=server_credentials
        )
        channel_options = (
            (
                "grpc.ssl_target_name_override",
                _SERVER_HOST_OVERRIDE,
            ),
        )
        self._channel = aio.secure_channel(
            self._server_address, channel_credentials, channel_options
        )
        self._stub = test_pb2_grpc.TestServiceStub(self._channel)

    async def tearDown(self):
        await self._channel.close()
        await self._server.stop(None)


class TestUnaryUnarySecureCall(_SecureCallMixin, AioTestBase):
    """unary_unary Calls made over a secure channel."""

    async def test_call_ok_over_secure_channel(self):
        call = self._stub.UnaryCall(messages_pb2.SimpleRequest())
        response = await call
        self.assertIsInstance(response, messages_pb2.SimpleResponse)
        self.assertEqual(await call.code(), grpc.StatusCode.OK)

    async def test_call_with_credentials(self):
        call_credentials = grpc.composite_call_credentials(
            grpc.access_token_call_credentials("abc"),
            grpc.access_token_call_credentials("def"),
        )
        call = self._stub.UnaryCall(
            messages_pb2.SimpleRequest(), credentials=call_credentials
        )
        response = await call

        self.assertIsInstance(response, messages_pb2.SimpleResponse)


class TestUnaryStreamSecureCall(_SecureCallMixin, AioTestBase):
    """unary_stream calls over a secure channel"""

    async def test_unary_stream_async_generator_secure(self):
        request = messages_pb2.StreamingOutputCallRequest()
        request.response_parameters.extend(
            messages_pb2.ResponseParameters(
                size=_RESPONSE_PAYLOAD_SIZE,
            )
            for _ in range(_NUM_STREAM_RESPONSES)
        )
        call_credentials = grpc.composite_call_credentials(
            grpc.access_token_call_credentials("abc"),
            grpc.access_token_call_credentials("def"),
        )
        call = self._stub.StreamingOutputCall(
            request, credentials=call_credentials
        )

        async for response in call:
            self.assertIsInstance(
                response, messages_pb2.StreamingOutputCallResponse
            )
            self.assertEqual(len(response.payload.body), _RESPONSE_PAYLOAD_SIZE)

        self.assertEqual(await call.code(), grpc.StatusCode.OK)


# Prepares the request that stream in a ping-pong manner.
_STREAM_OUTPUT_REQUEST_ONE_RESPONSE = messages_pb2.StreamingOutputCallRequest()
_STREAM_OUTPUT_REQUEST_ONE_RESPONSE.response_parameters.append(
    messages_pb2.ResponseParameters(size=_RESPONSE_PAYLOAD_SIZE)
)


class TestStreamStreamSecureCall(_SecureCallMixin, AioTestBase):
    _STREAM_ITERATIONS = 2

    async def test_async_generator_secure_channel(self):
        async def request_generator():
            for _ in range(self._STREAM_ITERATIONS):
                yield _STREAM_OUTPUT_REQUEST_ONE_RESPONSE

        call_credentials = grpc.composite_call_credentials(
            grpc.access_token_call_credentials("abc"),
            grpc.access_token_call_credentials("def"),
        )

        call = self._stub.FullDuplexCall(
            request_generator(), credentials=call_credentials
        )
        async for response in call:
            self.assertEqual(_RESPONSE_PAYLOAD_SIZE, len(response.payload.body))

        self.assertEqual(await call.code(), grpc.StatusCode.OK)


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main(verbosity=2)



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests of standard AuthMetadataPlugins."""

import collections
import logging
import threading
import unittest

from grpc import _auth


class MockGoogleCreds:
    def get_access_token(self):
        token = collections.namedtuple(
            "MockAccessTokenInfo", ("access_token", "expires_in")
        )
        token.access_token = "token"
        return token


class MockExceptionGoogleCreds:
    def get_access_token(self):
        raise Exception()


class GoogleCallCredentialsTest(unittest.TestCase):
    def test_google_call_credentials_success(self):
        callback_event = threading.Event()

        def mock_callback(metadata, error):
            self.assertEqual(metadata, (("authorization", "Bearer token"),))
            self.assertIsNone(error)
            callback_event.set()

        call_creds = _auth.GoogleCallCredentials(MockGoogleCreds())
        call_creds(None, mock_callback)
        self.assertTrue(callback_event.wait(1.0))

    def test_google_call_credentials_error(self):
        callback_event = threading.Event()

        def mock_callback(metadata, error):
            self.assertIsNotNone(error)
            callback_event.set()

        call_creds = _auth.GoogleCallCredentials(MockExceptionGoogleCreds())
        call_creds(None, mock_callback)
        self.assertTrue(callback_event.wait(1.0))


class AccessTokenAuthMetadataPluginTest(unittest.TestCase):
    def test_google_call_credentials_success(self):
        callback_event = threading.Event()

        def mock_callback(metadata, error):
            self.assertEqual(metadata, (("authorization", "Bearer token"),))
            self.assertIsNone(error)
            callback_event.set()

        metadata_plugin = _auth.AccessTokenAuthMetadataPlugin("token")
        metadata_plugin(None, mock_callback)
        self.assertTrue(callback_event.wait(1.0))


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Filter out tests based on file differences compared to merge target branch"""

import re
import subprocess


class TestSuite:
    """
    Contains label to identify job as belonging to this test suite and
    triggers to identify if changed files are relevant
    """

    def __init__(self, labels):
        """
        Build TestSuite to group tests based on labeling
        :param label: strings that should match a jobs's platform, config, language, or test group
        """
        self.triggers = []
        self.labels = labels

    def add_trigger(self, trigger):
        """
        Add a regex to list of triggers that determine if a changed file should run tests
        :param trigger: regex matching file relevant to tests
        """
        self.triggers.append(trigger)


# Create test suites
_CORE_TEST_SUITE = TestSuite(["c"])
_CPP_TEST_SUITE = TestSuite(["c++"])
_CSHARP_TEST_SUITE = TestSuite(["csharp"])
_NODE_TEST_SUITE = TestSuite(["grpc-node"])
_OBJC_TEST_SUITE = TestSuite(["objc"])
_PHP_TEST_SUITE = TestSuite(["php", "php7"])
_PYTHON_TEST_SUITE = TestSuite(["python"])
_RUBY_TEST_SUITE = TestSuite(["ruby"])
_LINUX_TEST_SUITE = TestSuite(["linux"])
_WINDOWS_TEST_SUITE = TestSuite(["windows"])
_MACOS_TEST_SUITE = TestSuite(["macos"])
_ALL_TEST_SUITES = [
    _CORE_TEST_SUITE,
    _CPP_TEST_SUITE,
    _CSHARP_TEST_SUITE,
    _NODE_TEST_SUITE,
    _OBJC_TEST_SUITE,
    _PHP_TEST_SUITE,
    _PYTHON_TEST_SUITE,
    _RUBY_TEST_SUITE,
    _LINUX_TEST_SUITE,
    _WINDOWS_TEST_SUITE,
    _MACOS_TEST_SUITE,
]

# Dictionary of allowlistable files where the key is a regex matching changed files
# and the value is a list of tests that should be run. An empty list means that
# the changed files should not trigger any tests. Any changed file that does not
# match any of these regexes will trigger all tests
# DO NOT CHANGE THIS UNLESS YOU KNOW WHAT YOU ARE DOING (be careful even if you do)
_ALLOWLIST_DICT = {
    r"^doc/": [],
    r"^examples/": [],
    r"^include/grpc\+\+/": [_CPP_TEST_SUITE],
    r"^include/grpcpp/": [_CPP_TEST_SUITE],
    r"^summerofcode/": [],
    r"^src/cpp/": [_CPP_TEST_SUITE],
    r"^src/csharp/": [_CSHARP_TEST_SUITE],
    r"^src/objective-c/": [_OBJC_TEST_SUITE],
    r"^src/php/": [_PHP_TEST_SUITE],
    r"^src/python/": [_PYTHON_TEST_SUITE],
    r"^src/ruby/": [_RUBY_TEST_SUITE],
    r"^templates/": [],
    r"^test/core/": [_CORE_TEST_SUITE, _CPP_TEST_SUITE],
    r"^test/cpp/": [_CPP_TEST_SUITE],
    r"^test/distrib/cpp/": [_CPP_TEST_SUITE],
    r"^test/distrib/csharp/": [_CSHARP_TEST_SUITE],
    r"^test/distrib/php/": [_PHP_TEST_SUITE],
    r"^test/distrib/python/": [_PYTHON_TEST_SUITE],
    r"^test/distrib/ruby/": [_RUBY_TEST_SUITE],
    r"^tools/run_tests/xds_k8s_test_driver/": [],
    r"^tools/internal_ci/linux/grpc_xds_k8s.*": [],
    r"^vsprojects/": [_WINDOWS_TEST_SUITE],
    r"composer\.json$": [_PHP_TEST_SUITE],
    r"config\.m4$": [_PHP_TEST_SUITE],
    r"CONTRIBUTING\.md$": [],
    r"Gemfile$": [_RUBY_TEST_SUITE],
    r"grpc\.def$": [_WINDOWS_TEST_SUITE],
    r"grpc\.gemspec$": [_RUBY_TEST_SUITE],
    r"gRPC\.podspec$": [_OBJC_TEST_SUITE],
    r"gRPC-Core\.podspec$": [_OBJC_TEST_SUITE],
    r"gRPC-ProtoRPC\.podspec$": [_OBJC_TEST_SUITE],
    r"gRPC-RxLibrary\.podspec$": [_OBJC_TEST_SUITE],
    r"BUILDING\.md$": [],
    r"LICENSE$": [],
    r"MANIFEST\.md$": [],
    r"package\.json$": [_PHP_TEST_SUITE],
    r"package\.xml$": [_PHP_TEST_SUITE],
    r"PATENTS$": [],
    r"PYTHON-MANIFEST\.in$": [_PYTHON_TEST_SUITE],
    r"README\.md$": [],
    r"requirements\.txt$": [_PYTHON_TEST_SUITE],
    r"setup\.cfg$": [_PYTHON_TEST_SUITE],
    r"setup\.py$": [_PYTHON_TEST_SUITE],
}

# Regex that combines all keys in _ALLOWLIST_DICT
_ALL_TRIGGERS = re.compile(f"({')|('.join(_ALLOWLIST_DICT)})")

# Add all triggers to their respective test suites
for trigger, test_suites in _ALLOWLIST_DICT.items():
    for test_suite in test_suites:
        test_suite.add_trigger(trigger)


def _get_changed_files(base_branch):
    """
    Get list of changed files between current branch and base of target merge branch
    """
    # Get file changes between branch and merge-base of specified branch
    # Not combined to be Windows friendly
    base_commit = (
        subprocess.check_output(["git", "merge-base", base_branch, "HEAD"])
        .decode("UTF-8")
        .rstrip()
    )
    return (
        subprocess.check_output(
            ["git", "diff", base_commit, "--name-only", "HEAD"]
        )
        .decode("UTF-8")
        .splitlines()
    )


def _can_skip_tests(file_names, triggers):
    """
    Determines if tests are skippable based on if all files do not match list of regexes
    :param file_names: list of changed files generated by _get_changed_files()
    :param triggers: list of regexes matching file name that indicates tests should be run
    :return: safe to skip tests
    """
    for file_name in file_names:
        if any(re.match(trigger, file_name) for trigger in triggers):
            return False
    return True


def _remove_irrelevant_tests(tests, skippable_labels):
    """
    Filters out tests by config or language - will not remove sanitizer tests
    :param tests: list of all tests generated by run_tests_matrix.py
    :param skippable_labels: list of languages and platforms with skippable tests
    :return: list of relevant tests
    """
    # test.labels[0] is platform and test.labels[2] is language
    # We skip a test if both are considered safe to skip
    return [
        test
        for test in tests
        if test.labels[0] not in skippable_labels
        or test.labels[2] not in skippable_labels
    ]


def affects_c_cpp(base_branch):
    """
    Determines if a pull request's changes affect C/C++. This function exists because
    there are pull request tests that only test C/C++ code
    :param base_branch: branch that a pull request is requesting to merge into
    :return: boolean indicating whether C/C++ changes are made in pull request
    """
    changed_files = _get_changed_files(base_branch)
    # Run all tests if any changed file is not in the allowlist dictionary
    for changed_file in changed_files:
        if not re.match(_ALL_TRIGGERS, changed_file):
            return True
    return not _can_skip_tests(
        changed_files, _CPP_TEST_SUITE.triggers + _CORE_TEST_SUITE.triggers
    )


def filter_tests(tests, base_branch):
    """
    Filters out tests that are safe to ignore
    :param tests: list of all tests generated by run_tests_matrix.py
    :return: list of relevant tests
    """
    print(
        "Finding file differences between gRPC %s branch and pull request...\n"
        % base_branch
    )
    changed_files = _get_changed_files(base_branch)
    for changed_file in changed_files:
        print("  %s" % changed_file)
    print("")

    # Run all tests if any changed file is not in the allowlist dictionary
    for changed_file in changed_files:
        if not re.match(_ALL_TRIGGERS, changed_file):
            return tests
    # Figure out which language and platform tests to run
    skippable_labels = []
    for test_suite in _ALL_TEST_SUITES:
        if _can_skip_tests(changed_files, test_suite.triggers):
            for label in test_suite.labels:
                print("  %s tests safe to skip" % label)
                skippable_labels.append(label)
    tests = _remove_irrelevant_tests(tests, skippable_labels)
    return tests



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: xds/type/v3/range.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'xds/type/v3/range.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17xds/type/v3/range.proto\x12\x0bxds.type.v3\"(\n\nInt64Range\x12\r\n\x05start\x18\x01 \x01(\x03\x12\x0b\n\x03\x65nd\x18\x02 \x01(\x03\"(\n\nInt32Range\x12\r\n\x05start\x18\x01 \x01(\x05\x12\x0b\n\x03\x65nd\x18\x02 \x01(\x05\")\n\x0b\x44oubleRange\x12\r\n\x05start\x18\x01 \x01(\x01\x12\x0b\n\x03\x65nd\x18\x02 \x01(\x01\x42J\n\x16\x63om.github.xds.type.v3B\nRangeProtoP\x01Z\"github.com/cncf/xds/go/xds/type/v3b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'xds.type.v3.range_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\026com.github.xds.type.v3B\nRangeProtoP\001Z\"github.com/cncf/xds/go/xds/type/v3'
  _globals['_INT64RANGE']._serialized_start=40
  _globals['_INT64RANGE']._serialized_end=80
  _globals['_INT32RANGE']._serialized_start=82
  _globals['_INT32RANGE']._serialized_end=122
  _globals['_DOUBLERANGE']._serialized_start=124
  _globals['_DOUBLERANGE']._serialized_end=165
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import route_guide_pb2 as route__guide__pb2


class RouteGuideStub:
    """Interface exported by the server.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetFeature = channel.unary_unary(
                '/routeguide.RouteGuide/GetFeature',
                request_serializer=route__guide__pb2.Point.SerializeToString,
                response_deserializer=route__guide__pb2.Feature.FromString,
                )
        self.ListFeatures = channel.unary_stream(
                '/routeguide.RouteGuide/ListFeatures',
                request_serializer=route__guide__pb2.Rectangle.SerializeToString,
                response_deserializer=route__guide__pb2.Feature.FromString,
                )
        self.RecordRoute = channel.stream_unary(
                '/routeguide.RouteGuide/RecordRoute',
                request_serializer=route__guide__pb2.Point.SerializeToString,
                response_deserializer=route__guide__pb2.RouteSummary.FromString,
                )
        self.RouteChat = channel.stream_stream(
                '/routeguide.RouteGuide/RouteChat',
                request_serializer=route__guide__pb2.RouteNote.SerializeToString,
                response_deserializer=route__guide__pb2.RouteNote.FromString,
                )


class RouteGuideServicer:
    """Interface exported by the server.
    """

    def GetFeature(self, request, context):
        """A simple RPC.

        Obtains the feature at a given position.

        A feature with an empty name is returned if there's no feature at the given
        position.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListFeatures(self, request, context):
        """A server-to-client streaming RPC.

        Obtains the Features available within the given Rectangle.  Results are
        streamed rather than returned at once (e.g. in a response message with a
        repeated field), as the rectangle may cover a large area and contain a
        huge number of features.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RecordRoute(self, request_iterator, context):
        """A client-to-server streaming RPC.

        Accepts a stream of Points on a route being traversed, returning a
        RouteSummary when traversal is completed.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RouteChat(self, request_iterator, context):
        """A Bidirectional streaming RPC.

        Accepts a stream of RouteNotes sent while a route is being traversed,
        while receiving other RouteNotes (e.g. from other users).
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_RouteGuideServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetFeature': grpc.unary_unary_rpc_method_handler(
                    servicer.GetFeature,
                    request_deserializer=route__guide__pb2.Point.FromString,
                    response_serializer=route__guide__pb2.Feature.SerializeToString,
            ),
            'ListFeatures': grpc.unary_stream_rpc_method_handler(
                    servicer.ListFeatures,
                    request_deserializer=route__guide__pb2.Rectangle.FromString,
                    response_serializer=route__guide__pb2.Feature.SerializeToString,
            ),
            'RecordRoute': grpc.stream_unary_rpc_method_handler(
                    servicer.RecordRoute,
                    request_deserializer=route__guide__pb2.Point.FromString,
                    response_serializer=route__guide__pb2.RouteSummary.SerializeToString,
            ),
            'RouteChat': grpc.stream_stream_rpc_method_handler(
                    servicer.RouteChat,
                    request_deserializer=route__guide__pb2.RouteNote.FromString,
                    response_serializer=route__guide__pb2.RouteNote.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'routeguide.RouteGuide', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class RouteGuide:
    """Interface exported by the server.
    """

    @staticmethod
    def GetFeature(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/routeguide.RouteGuide/GetFeature',
            route__guide__pb2.Point.SerializeToString,
            route__guide__pb2.Feature.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListFeatures(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/routeguide.RouteGuide/ListFeatures',
            route__guide__pb2.Rectangle.SerializeToString,
            route__guide__pb2.Feature.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RecordRoute(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_unary(request_iterator, target, '/routeguide.RouteGuide/RecordRoute',
            route__guide__pb2.Point.SerializeToString,
            route__guide__pb2.RouteSummary.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RouteChat(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/routeguide.RouteGuide/RouteChat',
            route__guide__pb2.RouteNote.SerializeToString,
            route__guide__pb2.RouteNote.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests behavior of the wait for connection API on client side."""

import asyncio
import datetime
import logging
from typing import Callable, Tuple
import unittest

import grpc
from grpc.experimental import aio

from src.proto.grpc.testing import messages_pb2
from src.proto.grpc.testing import test_pb2_grpc
from tests_aio.unit import _common
from tests_aio.unit._constants import UNREACHABLE_TARGET
from tests_aio.unit._test_base import AioTestBase
from tests_aio.unit._test_server import start_test_server

_REQUEST = b"\x01\x02\x03"
_TEST_METHOD = "/test/Test"

_NUM_STREAM_RESPONSES = 5
_REQUEST_PAYLOAD_SIZE = 7
_RESPONSE_PAYLOAD_SIZE = 42


class TestWaitForConnection(AioTestBase):
    """Tests if wait_for_connection raises connectivity issue."""

    async def setUp(self):
        address, self._server = await start_test_server()
        self._channel = aio.insecure_channel(address)
        self._phony_channel = aio.insecure_channel(UNREACHABLE_TARGET)
        self._stub = test_pb2_grpc.TestServiceStub(self._channel)

    async def tearDown(self):
        await self._phony_channel.close()
        await self._channel.close()
        await self._server.stop(None)

    async def test_unary_unary_ok(self):
        call = self._stub.UnaryCall(messages_pb2.SimpleRequest())

        # No exception raised and no message swallowed.
        await call.wait_for_connection()

        response = await call
        self.assertIsInstance(response, messages_pb2.SimpleResponse)

    async def test_unary_stream_ok(self):
        request = messages_pb2.StreamingOutputCallRequest()
        for _ in range(_NUM_STREAM_RESPONSES):
            request.response_parameters.append(
                messages_pb2.ResponseParameters(size=_RESPONSE_PAYLOAD_SIZE)
            )

        call = self._stub.StreamingOutputCall(request)

        # No exception raised and no message swallowed.
        await call.wait_for_connection()

        response_cnt = 0
        async for response in call:
            response_cnt += 1
            self.assertIs(
                type(response), messages_pb2.StreamingOutputCallResponse
            )
            self.assertEqual(_RESPONSE_PAYLOAD_SIZE, len(response.payload.body))

        self.assertEqual(_NUM_STREAM_RESPONSES, response_cnt)
        self.assertEqual(await call.code(), grpc.StatusCode.OK)

    async def test_stream_unary_ok(self):
        call = self._stub.StreamingInputCall()

        # No exception raised and no message swallowed.
        await call.wait_for_connection()

        payload = messages_pb2.Payload(body=b"\0" * _REQUEST_PAYLOAD_SIZE)
        request = messages_pb2.StreamingInputCallRequest(payload=payload)

        for _ in range(_NUM_STREAM_RESPONSES):
            await call.write(request)
        await call.done_writing()

        response = await call
        self.assertIsInstance(response, messages_pb2.StreamingInputCallResponse)
        self.assertEqual(
            _NUM_STREAM_RESPONSES * _REQUEST_PAYLOAD_SIZE,
            response.aggregated_payload_size,
        )

        self.assertEqual(await call.code(), grpc.StatusCode.OK)

    async def test_stream_stream_ok(self):
        call = self._stub.FullDuplexCall()

        # No exception raised and no message swallowed.
        await call.wait_for_connection()

        request = messages_pb2.StreamingOutputCallRequest()
        request.response_parameters.append(
            messages_pb2.ResponseParameters(size=_RESPONSE_PAYLOAD_SIZE)
        )

        for _ in range(_NUM_STREAM_RESPONSES):
            await call.write(request)
            response = await call.read()
            self.assertIsInstance(
                response, messages_pb2.StreamingOutputCallResponse
            )
            self.assertEqual(_RESPONSE_PAYLOAD_SIZE, len(response.payload.body))

        await call.done_writing()

        self.assertEqual(grpc.StatusCode.OK, await call.code())

    async def test_unary_unary_error(self):
        call = self._phony_channel.unary_unary(_TEST_METHOD)(_REQUEST)

        with self.assertRaises(aio.AioRpcError) as exception_context:
            await call.wait_for_connection()
        rpc_error = exception_context.exception
        self.assertEqual(grpc.StatusCode.UNAVAILABLE, rpc_error.code())

    async def test_unary_stream_error(self):
        call = self._phony_channel.unary_stream(_TEST_METHOD)(_REQUEST)

        with self.assertRaises(aio.AioRpcError) as exception_context:
            await call.wait_for_connection()
        rpc_error = exception_context.exception
        self.assertEqual(grpc.StatusCode.UNAVAILABLE, rpc_error.code())

    async def test_stream_unary_error(self):
        call = self._phony_channel.stream_unary(_TEST_METHOD)()

        with self.assertRaises(aio.AioRpcError) as exception_context:
            await call.wait_for_connection()
        rpc_error = exception_context.exception
        self.assertEqual(grpc.StatusCode.UNAVAILABLE, rpc_error.code())

    async def test_stream_stream_error(self):
        call = self._phony_channel.stream_stream(_TEST_METHOD)()

        with self.assertRaises(aio.AioRpcError) as exception_context:
            await call.wait_for_connection()
        rpc_error = exception_context.exception
        self.assertEqual(grpc.StatusCode.UNAVAILABLE, rpc_error.code())


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main(verbosity=2)



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The Python implementation of the gRPC route guide client."""

from __future__ import print_function

import logging
import random

import grpc
import route_guide_pb2
import route_guide_pb2_grpc
import route_guide_resources


def make_route_note(message, latitude, longitude):
    return route_guide_pb2.RouteNote(
        message=message,
        location=route_guide_pb2.Point(latitude=latitude, longitude=longitude),
    )


def format_point(point):
    # not delegating in point.__str__ because it is an empty string when its
    # values are zero. In addition, it puts a newline between the fields.
    return "latitude: %d, longitude: %d" % (point.latitude, point.longitude)


def guide_get_one_feature(stub, point):
    feature = stub.GetFeature(point)
    if not feature.location:
        print("Server returned incomplete feature")
        return

    if feature.name:
        print(
            "Feature called %r at %s"
            % (feature.name, format_point(feature.location))
        )
    else:
        print("Found no feature at %s" % format_point(feature.location))


def guide_get_feature(stub):
    guide_get_one_feature(
        stub, route_guide_pb2.Point(latitude=409146138, longitude=-746188906)
    )
    guide_get_one_feature(stub, route_guide_pb2.Point(latitude=0, longitude=0))


def guide_list_features(stub):
    rectangle = route_guide_pb2.Rectangle(
        lo=route_guide_pb2.Point(latitude=400000000, longitude=-750000000),
        hi=route_guide_pb2.Point(latitude=420000000, longitude=-730000000),
    )
    print("Looking for features between 40, -75 and 42, -73")

    features = stub.ListFeatures(rectangle)

    for feature in features:
        print(
            "Feature called %r at %s"
            % (feature.name, format_point(feature.location))
        )


def generate_route(feature_list):
    for _ in range(0, 10):
        random_feature = random.choice(feature_list)
        print("Visiting point %s" % format_point(random_feature.location))
        yield random_feature.location


def guide_record_route(stub):
    feature_list = route_guide_resources.read_route_guide_database()

    route_iterator = generate_route(feature_list)
    route_summary = stub.RecordRoute(route_iterator)
    print("Finished trip with %s points " % route_summary.point_count)
    print("Passed %s features " % route_summary.feature_count)
    print("Travelled %s meters " % route_summary.distance)
    print("It took %s seconds " % route_summary.elapsed_time)


def generate_messages():
    messages = [
        make_route_note("First message", 0, 0),
        make_route_note("Second message", 0, 1),
        make_route_note("Third message", 1, 0),
        make_route_note("Fourth message", 0, 0),
        make_route_note("Fifth message", 1, 0),
    ]
    for msg in messages:
        print("Sending %s at %s" % (msg.message, format_point(msg.location)))
        yield msg


def guide_route_chat(stub):
    responses = stub.RouteChat(generate_messages())
    for response in responses:
        print(
            "Received message %s at %s"
            % (response.message, format_point(response.location))
        )


def run():
    # NOTE(gRPC Python Team): .close() is possible on a channel and should be
    # used in circumstances in which the with statement does not fit the needs
    # of the code.
    with grpc.insecure_channel("localhost:50051") as channel:
        stub = route_guide_pb2_grpc.RouteGuideStub(channel)
        print("-------------- GetFeature --------------")
        guide_get_feature(stub)
        print("-------------- ListFeatures --------------")
        guide_list_features(stub)
        print("-------------- RecordRoute --------------")
        guide_record_route(stub)
        print("-------------- RouteChat --------------")
        guide_route_chat(stub)


if __name__ == "__main__":
    logging.basicConfig()
    run()



#!/usr/bin/env python3

# Copyright 2023 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import errno
import os
import os.path
import pprint
import shutil
import subprocess
import sys
import traceback

# the template for the content of observability_lib_deps.py
DEPS_FILE_CONTENT = """
# Copyright 2023 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# AUTO-GENERATED BY make_grpcio_observability.py!
CC_FILES={cc_files}

CC_INCLUDES={cc_includes}
"""

# maps bazel reference to actual path
BAZEL_REFERENCE_LINK = [
    ("@com_google_absl//", "third_party/abseil-cpp/"),
    ("@com_google_protobuf//upb/", "third_party/protobuf/upb/"),
    ("@com_google_protobuf//third_party/", "third_party/protobuf/third_party/"),
    ("//src", "grpc_root/src"),
]

# maps bazel reference to a proto to actual path
BAZEL_PROTO_REFERENCE_LINK = [
    ("//src/proto/", "grpc_root/src/core/ext/upb-gen/src/proto/"),
    (
        "@com_google_protobuf//src/google/",
        "grpc_root/src/core/ext/upb-gen/google/",
    ),
]

ABSL_INCLUDE = (os.path.join("third_party", "abseil-cpp"),)

UPB_GEN_INCLUDE = (os.path.join("grpc_root", "src", "core", "ext", "upb-gen"),)
UPB_DEFS_GEN_INCLUDE = (
    os.path.join("grpc_root", "src", "core", "ext", "upbdefs-gen"),
)

# Exclude conditional dependencies that are not needed by gRPC.
# Required files can be found at third_party/protobuf/src/file_lists.cmake
UPB_EXCLUDE_CC_FILES_PATTERN = "third_party/protobuf/upb/wire/decode_fast"
UPB_EXCLUDE_CC_FILES_EXCEPTIONS = [
    "third_party/protobuf/upb/wire/decode_fast/select.c",
]

PROTOBUF_INCLUDE = (os.path.join("third_party", "protobuf"),)
PROTOBUF_UTF8_RANGE_INCLUDE = (
    os.path.join("third_party", "protobuf", "third_party", "utf8_range"),
)

# will be added to include path when building grpcio_observability
EXTENSION_INCLUDE_DIRECTORIES = (
    ABSL_INCLUDE
    + UPB_GEN_INCLUDE
    + UPB_DEFS_GEN_INCLUDE
    + PROTOBUF_INCLUDE
    + PROTOBUF_UTF8_RANGE_INCLUDE
)

CC_INCLUDES = list(EXTENSION_INCLUDE_DIRECTORIES)

# the target directory is relative to the grpcio_observability package root.
GRPCIO_OBSERVABILITY_ROOT_PREFIX = "src/python/grpcio_observability/"

# Pairs of (source, target) directories to copy
# from the grpc repo root to the grpcio_observability build root.
COPY_FILES_SOURCE_TARGET_PAIRS = [
    ("include", "grpc_root/include"),
    ("third_party/abseil-cpp/absl", "third_party/abseil-cpp/absl"),
    ("third_party/protobuf/upb", "third_party/protobuf/upb"),
    (
        "third_party/protobuf/third_party/utf8_range",
        "third_party/protobuf/third_party/utf8_range",
    ),
    ("src/core", "grpc_root/src/core"),
]

# grpc repo root
GRPC_ROOT = os.path.abspath(
    os.path.join(os.path.dirname(os.path.abspath(__file__)), "..", "..", "..")
)


# the file to generate
GRPC_PYTHON_OBSERVABILITY_LIB_DEPS = os.path.join(
    GRPC_ROOT,
    "src",
    "python",
    "grpcio_observability",
    "observability_lib_deps.py",
)

# the script to run for getting dependencies
BAZEL_DEPS = os.path.join(
    GRPC_ROOT, "tools", "distrib", "python", "bazel_deps.sh"
)

# the bazel target to scrape to get list of sources for the build
BAZEL_DEPS_QUERIES = [
    "//src/core:experiments",
    "//src/core:slice",
    "//src/core:ref_counted_string",
    "//src/core:instrument",
]


def _bazel_query(query):
    """Runs 'bazel query' to collect source file info."""
    print('Running "bazel query %s"' % query)
    output = subprocess.check_output([BAZEL_DEPS, query])
    return output.decode("ascii").splitlines()


def _pretty_print_list(items):
    """Pretty print python list"""
    formatted = pprint.pformat(items, indent=4)
    # add newline after opening bracket (and fix indent of the next line)
    if formatted.startswith("["):
        formatted = formatted[0] + "\n " + formatted[1:]
    # add newline before closing bracket
    if formatted.endswith("]"):
        formatted = formatted[:-1] + "\n" + formatted[-1]
    return formatted


def _bazel_name_to_file_path(name):
    """Transform bazel reference to source file name."""
    for link in BAZEL_REFERENCE_LINK:
        if name.startswith(link[0]):
            filepath = link[1] + name[len(link[0]) :].replace(":", "/")
            return filepath
    return None


# TODO(sergiitk): remove after all CI run on python3.9+, use str.removeprefix
def _removeprefix(input: str, prefix: str, /) -> str:
    if input.startswith(prefix):
        return input[len(prefix) :]
    else:
        return input[:]


def _bazel_proto_name_to_file_path(proto_target: str):
    """Transform bazel proto target to local file name."""
    for bazel_prefix, local_path in BAZEL_PROTO_REFERENCE_LINK:
        if proto_target.startswith(bazel_prefix):
            normalized_name = (
                _removeprefix(proto_target, bazel_prefix)
                .lstrip("/")
                .replace(":", "/")
            )
            new_name = normalized_name.replace(".proto", ".upb_minitable.c")
            return f"{local_path.rstrip('/')}/{new_name}"
    return None


def _generate_deps_file_content():
    """Returns the data structure with dependencies of protoc as python code."""
    cc_files_output = []
    for query in BAZEL_DEPS_QUERIES:
        cc_files_output += _bazel_query(query)

    # Collect .cc files (that will be later included in the native extension build)
    cc_files = set()
    for name in cc_files_output:
        if name.endswith(".proto"):
            filepath = _bazel_proto_name_to_file_path(name)
            if filepath and os.path.exists(
                _removeprefix(filepath, "grpc_root/")
            ):
                cc_files.add(filepath)
        if name.endswith(".cc") or (
            name.endswith(".c")
            and not name.endswith((".upb.c", ".upb_minitable.c"))
        ):
            filepath = _bazel_name_to_file_path(name)
            if filepath:
                cc_files.add(filepath)

    # Exclude certain conditional dependencies that don't build properly
    # from upb.
    def _should_include(cc_file):
        if UPB_EXCLUDE_CC_FILES_PATTERN not in cc_file:
            return True
        if cc_file in UPB_EXCLUDE_CC_FILES_EXCEPTIONS:
            return True
        return False

    cc_files = [f for f in cc_files if _should_include(f)]

    deps_file_content = DEPS_FILE_CONTENT.format(
        cc_files=_pretty_print_list(sorted(list(cc_files))),
        cc_includes=_pretty_print_list(CC_INCLUDES),
    )
    return deps_file_content


def _copy_source_tree(source, target):
    """Copies source directory to a given target directory."""
    print("Copying contents of %s to %s" % (source, target))
    for source_dir, _, files in os.walk(source):
        target_dir = os.path.abspath(
            os.path.join(target, os.path.relpath(source_dir, source))
        )
        try:
            os.makedirs(target_dir)
        except OSError as error:
            if error.errno != errno.EEXIST:
                raise
        for relative_file in files:
            source_file = os.path.abspath(
                os.path.join(source_dir, relative_file)
            )
            target_file = os.path.abspath(
                os.path.join(target_dir, relative_file)
            )
            shutil.copyfile(source_file, target_file)


def main():
    os.chdir(GRPC_ROOT)

    # Step 1:
    # In order to be able to build the grpcio_observability package, we need the source
    # code for the plugins and its dependencies to be available under the build root of
    # the grpcio_observability package.
    # So we simply copy all the necessary files where the build will expect them to be.
    for source, target in COPY_FILES_SOURCE_TARGET_PAIRS:
        # convert the slashes in the relative path to platform-specific path dividers.
        # All paths are relative to GRPC_ROOT
        source_abs = os.path.join(GRPC_ROOT, os.path.join(*source.split("/")))
        # for targets, add grpcio_observability root prefix
        target = GRPCIO_OBSERVABILITY_ROOT_PREFIX + target
        target_abs = os.path.join(GRPC_ROOT, os.path.join(*target.split("/")))
        _copy_source_tree(source_abs, target_abs)
    print(
        "The necessary source files were copied under the grpcio_observability package root."
    )

    # Step 2:
    # Extract build metadata from bazel build (by running "bazel query")
    # and populate the observability_lib_deps.py file with python-readable data structure
    # that will be used by grpcio_observability's setup.py.
    try:
        print('Invoking "bazel query" to gather the dependencies.')
        observability_lib_deps_content = _generate_deps_file_content()
    except Exception as error:
        # We allow this script to succeed even if we couldn't get the dependencies,
        # as then we can assume that even without a successful bazel run the
        # dependencies currently in source control are 'good enough'.
        sys.stderr.write("Got non-fatal error:\n")
        traceback.print_exc(file=sys.stderr)
        return
    # If we successfully got the dependencies, truncate and rewrite the deps file.
    with open(GRPC_PYTHON_OBSERVABILITY_LIB_DEPS, "w") as deps_file:
        deps_file.write(observability_lib_deps_content)
    print('File "%s" updated.' % GRPC_PYTHON_OBSERVABILITY_LIB_DEPS)
    print("Done.")


if __name__ == "__main__":
    main()



# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Reference implementation for health checking in gRPC Python."""

import asyncio
import collections
from typing import MutableMapping

import grpc
from grpc_health.v1 import health_pb2 as _health_pb2
from grpc_health.v1 import health_pb2_grpc as _health_pb2_grpc


class HealthServicer(_health_pb2_grpc.HealthServicer):
    """An AsyncIO implementation of health checking servicer."""

    _server_status: MutableMapping[
        str, "_health_pb2.HealthCheckResponse.ServingStatus"
    ]
    _server_watchers: MutableMapping[str, asyncio.Condition]
    _gracefully_shutting_down: bool

    def __init__(self) -> None:
        self._server_status = {"": _health_pb2.HealthCheckResponse.SERVING}
        self._server_watchers = collections.defaultdict(asyncio.Condition)
        self._gracefully_shutting_down = False

    async def Check(
        self, request: _health_pb2.HealthCheckRequest, context
    ) -> None:
        status = self._server_status.get(request.service)

        if status is None:
            await context.abort(grpc.StatusCode.NOT_FOUND)
        else:
            return _health_pb2.HealthCheckResponse(status=status)

    async def Watch(
        self, request: _health_pb2.HealthCheckRequest, context
    ) -> None:
        condition = self._server_watchers[request.service]
        last_status = None
        try:
            async with condition:
                while True:
                    status = self._server_status.get(
                        request.service,
                        _health_pb2.HealthCheckResponse.SERVICE_UNKNOWN,
                    )

                    # NOTE(lidiz) If the observed status is the same, it means
                    # there are missing intermediate statuses. It's considered
                    # acceptable since peer only interested in eventual status.
                    if status != last_status:
                        # Responds with current health state
                        await context.write(
                            _health_pb2.HealthCheckResponse(status=status)
                        )

                    # Records the last sent status
                    last_status = status

                    # Polling on health state changes
                    await condition.wait()
        finally:
            if request.service in self._server_watchers:
                del self._server_watchers[request.service]

    async def _set(
        self,
        service: str,
        status: _health_pb2.HealthCheckResponse.ServingStatus,
    ) -> None:
        if service in self._server_watchers:
            condition = self._server_watchers.get(service)
            async with condition:
                self._server_status[service] = status
                condition.notify_all()
        else:
            self._server_status[service] = status

    async def set(
        self,
        service: str,
        status: _health_pb2.HealthCheckResponse.ServingStatus,
    ) -> None:
        """Sets the status of a service.

        Args:
          service: string, the name of the service.
          status: HealthCheckResponse.status enum value indicating the status of
            the service
        """
        if self._gracefully_shutting_down:
            return
        await self._set(service, status)

    async def enter_graceful_shutdown(self) -> None:
        """Permanently sets the status of all services to NOT_SERVING.

        This should be invoked when the server is entering a graceful shutdown
        period. After this method is invoked, future attempts to set the status
        of a service will be ignored.
        """
        if self._gracefully_shutting_down:
            return
        self._gracefully_shutting_down = True
        for service in self._server_status:
            await self._set(
                service, _health_pb2.HealthCheckResponse.NOT_SERVING
            )



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test-appropriate entry points into the gRPC Python Beta API."""

import grpc
from grpc.beta import implementations


def not_really_secure_channel(
    host, port, channel_credentials, server_host_override
):
    """Creates an insecure Channel to a remote host.

    Args:
      host: The name of the remote host to which to connect.
      port: The port of the remote host to which to connect.
      channel_credentials: The implementations.ChannelCredentials with which to
        connect.
      server_host_override: The target name used for SSL host name checking.

    Returns:
      An implementations.Channel to the remote host through which RPCs may be
        conducted.
    """
    target = "%s:%d" % (host, port)
    channel = grpc.secure_channel(
        target,
        channel_credentials,
        (
            (
                "grpc.ssl_target_name_override",
                server_host_override,
            ),
        ),
    )
    return implementations.Channel(channel)



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test of RPCs made against gRPC Python's application-layer API."""

from concurrent import futures
import itertools
import logging
import threading
import unittest

import grpc
from grpc.framework.foundation import logging_pool

from tests.unit._rpc_test_helpers import (
    stream_stream_non_blocking_multi_callable,
)
from tests.unit._rpc_test_helpers import (
    unary_stream_non_blocking_multi_callable,
)
from tests.unit._rpc_test_helpers import BaseRPCTest
from tests.unit._rpc_test_helpers import Callback
from tests.unit._rpc_test_helpers import TIMEOUT_SHORT
from tests.unit._rpc_test_helpers import stream_stream_multi_callable
from tests.unit._rpc_test_helpers import stream_unary_multi_callable
from tests.unit._rpc_test_helpers import unary_stream_multi_callable
from tests.unit._rpc_test_helpers import unary_unary_multi_callable
from tests.unit.framework.common import test_constants


class RPCPart2Test(BaseRPCTest, unittest.TestCase):
    def testDefaultThreadPoolIsUsed(self):
        self._consume_one_stream_response_unary_request(
            unary_stream_multi_callable(self._channel)
        )
        self.assertFalse(self._thread_pool.was_used())

    def testExperimentalThreadPoolIsUsed(self):
        self._consume_one_stream_response_unary_request(
            unary_stream_non_blocking_multi_callable(self._channel)
        )
        self.assertTrue(self._thread_pool.was_used())

    def testUnrecognizedMethod(self):
        request = b"abc"

        with self.assertRaises(grpc.RpcError) as exception_context:
            self._channel.unary_unary("NoSuchMethod")(request)

        self.assertEqual(
            grpc.StatusCode.UNIMPLEMENTED, exception_context.exception.code()
        )

    def testSuccessfulUnaryRequestBlockingUnaryResponse(self):
        request = b"\x07\x08"
        expected_response = self._handler.handle_unary_unary(request, None)

        multi_callable = unary_unary_multi_callable(self._channel)
        response = multi_callable(
            request,
            metadata=(("test", "SuccessfulUnaryRequestBlockingUnaryResponse"),),
        )

        self.assertEqual(expected_response, response)

    def testSuccessfulUnaryRequestBlockingUnaryResponseWithCall(self):
        request = b"\x07\x08"
        expected_response = self._handler.handle_unary_unary(request, None)

        multi_callable = unary_unary_multi_callable(self._channel)
        response, call = multi_callable.with_call(
            request,
            metadata=(
                ("test", "SuccessfulUnaryRequestBlockingUnaryResponseWithCall"),
            ),
        )

        self.assertEqual(expected_response, response)
        self.assertIs(grpc.StatusCode.OK, call.code())
        self.assertEqual("", call.debug_error_string())

    def testSuccessfulUnaryRequestFutureUnaryResponse(self):
        request = b"\x07\x08"
        expected_response = self._handler.handle_unary_unary(request, None)

        multi_callable = unary_unary_multi_callable(self._channel)
        response_future = multi_callable.future(
            request,
            metadata=(("test", "SuccessfulUnaryRequestFutureUnaryResponse"),),
        )
        response = response_future.result()

        self.assertIsInstance(response_future, grpc.Future)
        self.assertIsInstance(response_future, grpc.Call)
        self.assertEqual(expected_response, response)
        self.assertIsNone(response_future.exception())
        self.assertIsNone(response_future.traceback())

    def testSuccessfulUnaryRequestStreamResponse(self):
        request = b"\x37\x58"
        expected_responses = tuple(
            self._handler.handle_unary_stream(request, None)
        )

        multi_callable = unary_stream_multi_callable(self._channel)
        response_iterator = multi_callable(
            request,
            metadata=(("test", "SuccessfulUnaryRequestStreamResponse"),),
        )
        responses = tuple(response_iterator)

        self.assertSequenceEqual(expected_responses, responses)

    def testSuccessfulStreamRequestBlockingUnaryResponse(self):
        requests = tuple(
            b"\x07\x08" for _ in range(test_constants.STREAM_LENGTH)
        )
        expected_response = self._handler.handle_stream_unary(
            iter(requests), None
        )
        request_iterator = iter(requests)

        multi_callable = stream_unary_multi_callable(self._channel)
        response = multi_callable(
            request_iterator,
            metadata=(
                ("test", "SuccessfulStreamRequestBlockingUnaryResponse"),
            ),
        )

        self.assertEqual(expected_response, response)

    def testSuccessfulStreamRequestBlockingUnaryResponseWithCall(self):
        requests = tuple(
            b"\x07\x08" for _ in range(test_constants.STREAM_LENGTH)
        )
        expected_response = self._handler.handle_stream_unary(
            iter(requests), None
        )
        request_iterator = iter(requests)

        multi_callable = stream_unary_multi_callable(self._channel)
        response, call = multi_callable.with_call(
            request_iterator,
            metadata=(
                (
                    "test",
                    "SuccessfulStreamRequestBlockingUnaryResponseWithCall",
                ),
            ),
        )

        self.assertEqual(expected_response, response)
        self.assertIs(grpc.StatusCode.OK, call.code())

    def testSuccessfulStreamRequestFutureUnaryResponse(self):
        requests = tuple(
            b"\x07\x08" for _ in range(test_constants.STREAM_LENGTH)
        )
        expected_response = self._handler.handle_stream_unary(
            iter(requests), None
        )
        request_iterator = iter(requests)

        multi_callable = stream_unary_multi_callable(self._channel)
        response_future = multi_callable.future(
            request_iterator,
            metadata=(("test", "SuccessfulStreamRequestFutureUnaryResponse"),),
        )
        response = response_future.result()

        self.assertEqual(expected_response, response)
        self.assertIsNone(response_future.exception())
        self.assertIsNone(response_future.traceback())

    def testSuccessfulStreamRequestStreamResponse(self):
        requests = tuple(
            b"\x77\x58" for _ in range(test_constants.STREAM_LENGTH)
        )

        expected_responses = tuple(
            self._handler.handle_stream_stream(iter(requests), None)
        )
        request_iterator = iter(requests)

        multi_callable = stream_stream_multi_callable(self._channel)
        response_iterator = multi_callable(
            request_iterator,
            metadata=(("test", "SuccessfulStreamRequestStreamResponse"),),
        )
        responses = tuple(response_iterator)

        self.assertSequenceEqual(expected_responses, responses)

    def testSequentialInvocations(self):
        first_request = b"\x07\x08"
        second_request = b"\x0809"
        expected_first_response = self._handler.handle_unary_unary(
            first_request, None
        )
        expected_second_response = self._handler.handle_unary_unary(
            second_request, None
        )

        multi_callable = unary_unary_multi_callable(self._channel)
        first_response = multi_callable(
            first_request, metadata=(("test", "SequentialInvocations"),)
        )
        second_response = multi_callable(
            second_request, metadata=(("test", "SequentialInvocations"),)
        )

        self.assertEqual(expected_first_response, first_response)
        self.assertEqual(expected_second_response, second_response)

    def testConcurrentBlockingInvocations(self):
        pool = logging_pool.pool(test_constants.THREAD_CONCURRENCY)
        requests = tuple(
            b"\x07\x08" for _ in range(test_constants.STREAM_LENGTH)
        )
        expected_response = self._handler.handle_stream_unary(
            iter(requests), None
        )
        expected_responses = [
            expected_response
        ] * test_constants.THREAD_CONCURRENCY
        response_futures = [None] * test_constants.THREAD_CONCURRENCY

        multi_callable = stream_unary_multi_callable(self._channel)
        for index in range(test_constants.THREAD_CONCURRENCY):
            request_iterator = iter(requests)
            response_future = pool.submit(
                multi_callable,
                request_iterator,
                metadata=(("test", "ConcurrentBlockingInvocations"),),
            )
            response_futures[index] = response_future
        responses = tuple(
            response_future.result() for response_future in response_futures
        )

        pool.shutdown(wait=True)
        self.assertSequenceEqual(expected_responses, responses)

    def testConcurrentFutureInvocations(self):
        requests = tuple(
            b"\x07\x08" for _ in range(test_constants.STREAM_LENGTH)
        )
        expected_response = self._handler.handle_stream_unary(
            iter(requests), None
        )
        expected_responses = [
            expected_response
        ] * test_constants.THREAD_CONCURRENCY
        response_futures = [None] * test_constants.THREAD_CONCURRENCY

        multi_callable = stream_unary_multi_callable(self._channel)
        for index in range(test_constants.THREAD_CONCURRENCY):
            request_iterator = iter(requests)
            response_future = multi_callable.future(
                request_iterator,
                metadata=(("test", "ConcurrentFutureInvocations"),),
            )
            response_futures[index] = response_future
        responses = tuple(
            response_future.result() for response_future in response_futures
        )

        self.assertSequenceEqual(expected_responses, responses)

    def testWaitingForSomeButNotAllConcurrentFutureInvocations(self):
        pool = logging_pool.pool(test_constants.THREAD_CONCURRENCY)
        request = b"\x67\x68"
        expected_response = self._handler.handle_unary_unary(request, None)
        response_futures = [None] * test_constants.THREAD_CONCURRENCY
        lock = threading.Lock()
        test_is_running_cell = [True]

        def wrap_future(future):
            def wrap():
                try:
                    return future.result()
                except grpc.RpcError:
                    with lock:
                        if test_is_running_cell[0]:
                            raise
                    return None

            return wrap

        multi_callable = unary_unary_multi_callable(self._channel)
        for index in range(test_constants.THREAD_CONCURRENCY):
            inner_response_future = multi_callable.future(
                request,
                metadata=(
                    (
                        "test",
                        "WaitingForSomeButNotAllConcurrentFutureInvocations",
                    ),
                ),
            )
            outer_response_future = pool.submit(
                wrap_future(inner_response_future)
            )
            response_futures[index] = outer_response_future

        some_completed_response_futures_iterator = itertools.islice(
            futures.as_completed(response_futures),
            test_constants.THREAD_CONCURRENCY // 2,
        )
        for response_future in some_completed_response_futures_iterator:
            self.assertEqual(expected_response, response_future.result())
        with lock:
            test_is_running_cell[0] = False

    def testConsumingOneStreamResponseUnaryRequest(self):
        self._consume_one_stream_response_unary_request(
            unary_stream_multi_callable(self._channel)
        )

    def testConsumingOneStreamResponseUnaryRequestNonBlocking(self):
        self._consume_one_stream_response_unary_request(
            unary_stream_non_blocking_multi_callable(self._channel)
        )

    def testConsumingSomeButNotAllStreamResponsesUnaryRequest(self):
        self._consume_some_but_not_all_stream_responses_unary_request(
            unary_stream_multi_callable(self._channel)
        )

    def testConsumingSomeButNotAllStreamResponsesUnaryRequestNonBlocking(self):
        self._consume_some_but_not_all_stream_responses_unary_request(
            unary_stream_non_blocking_multi_callable(self._channel)
        )

    def testConsumingSomeButNotAllStreamResponsesStreamRequest(self):
        self._consume_some_but_not_all_stream_responses_stream_request(
            stream_stream_multi_callable(self._channel)
        )

    def testConsumingSomeButNotAllStreamResponsesStreamRequestNonBlocking(self):
        self._consume_some_but_not_all_stream_responses_stream_request(
            stream_stream_non_blocking_multi_callable(self._channel)
        )

    def testConsumingTooManyStreamResponsesStreamRequest(self):
        self._consume_too_many_stream_responses_stream_request(
            stream_stream_multi_callable(self._channel)
        )

    def testConsumingTooManyStreamResponsesStreamRequestNonBlocking(self):
        self._consume_too_many_stream_responses_stream_request(
            stream_stream_non_blocking_multi_callable(self._channel)
        )

    def testCancelledUnaryRequestUnaryResponse(self):
        request = b"\x07\x17"

        multi_callable = unary_unary_multi_callable(self._channel)
        with self._control.pause():
            response_future = multi_callable.future(
                request,
                metadata=(("test", "CancelledUnaryRequestUnaryResponse"),),
            )
            response_future.cancel()

        self.assertIs(grpc.StatusCode.CANCELLED, response_future.code())
        self.assertTrue(response_future.cancelled())
        with self.assertRaises(grpc.FutureCancelledError):
            response_future.result()
        with self.assertRaises(grpc.FutureCancelledError):
            response_future.exception()
        with self.assertRaises(grpc.FutureCancelledError):
            response_future.traceback()

    def testCancelledUnaryRequestStreamResponse(self):
        self._cancelled_unary_request_stream_response(
            unary_stream_multi_callable(self._channel)
        )

    def testCancelledUnaryRequestStreamResponseNonBlocking(self):
        self._cancelled_unary_request_stream_response(
            unary_stream_non_blocking_multi_callable(self._channel)
        )

    def testCancelledStreamRequestUnaryResponse(self):
        requests = tuple(
            b"\x07\x08" for _ in range(test_constants.STREAM_LENGTH)
        )
        request_iterator = iter(requests)

        multi_callable = stream_unary_multi_callable(self._channel)
        with self._control.pause():
            response_future = multi_callable.future(
                request_iterator,
                metadata=(("test", "CancelledStreamRequestUnaryResponse"),),
            )
            self._control.block_until_paused()
            response_future.cancel()

        self.assertIs(grpc.StatusCode.CANCELLED, response_future.code())
        self.assertTrue(response_future.cancelled())
        with self.assertRaises(grpc.FutureCancelledError):
            response_future.result()
        with self.assertRaises(grpc.FutureCancelledError):
            response_future.exception()
        with self.assertRaises(grpc.FutureCancelledError):
            response_future.traceback()
        self.assertIsNotNone(response_future.initial_metadata())
        self.assertIsNotNone(response_future.details())
        self.assertIsNotNone(response_future.trailing_metadata())

    def testCancelledStreamRequestStreamResponse(self):
        self._cancelled_stream_request_stream_response(
            stream_stream_multi_callable(self._channel)
        )

    def testCancelledStreamRequestStreamResponseNonBlocking(self):
        self._cancelled_stream_request_stream_response(
            stream_stream_non_blocking_multi_callable(self._channel)
        )

    def testExpiredUnaryRequestBlockingUnaryResponse(self):
        request = b"\x07\x17"

        multi_callable = unary_unary_multi_callable(self._channel)
        with self._control.pause():
            with self.assertRaises(grpc.RpcError) as exception_context:
                multi_callable.with_call(
                    request,
                    timeout=TIMEOUT_SHORT,
                    metadata=(
                        ("test", "ExpiredUnaryRequestBlockingUnaryResponse"),
                    ),
                )

        self.assertIsInstance(exception_context.exception, grpc.Call)
        self.assertIsNotNone(exception_context.exception.initial_metadata())
        self.assertIs(
            grpc.StatusCode.DEADLINE_EXCEEDED,
            exception_context.exception.code(),
        )
        self.assertIsNotNone(exception_context.exception.details())
        self.assertIsNotNone(exception_context.exception.trailing_metadata())

    def testExpiredUnaryRequestFutureUnaryResponse(self):
        request = b"\x07\x17"
        callback = Callback()

        multi_callable = unary_unary_multi_callable(self._channel)
        with self._control.pause():
            response_future = multi_callable.future(
                request,
                timeout=TIMEOUT_SHORT,
                metadata=(("test", "ExpiredUnaryRequestFutureUnaryResponse"),),
            )
            response_future.add_done_callback(callback)
            value_passed_to_callback = callback.value()

        self.assertIs(response_future, value_passed_to_callback)
        self.assertIsNotNone(response_future.initial_metadata())
        self.assertIs(grpc.StatusCode.DEADLINE_EXCEEDED, response_future.code())
        self.assertIsNotNone(response_future.details())
        self.assertIsNotNone(response_future.trailing_metadata())
        with self.assertRaises(grpc.RpcError) as exception_context:
            response_future.result()
        self.assertIs(
            grpc.StatusCode.DEADLINE_EXCEEDED,
            exception_context.exception.code(),
        )
        self.assertIsInstance(response_future.exception(), grpc.RpcError)
        self.assertIsNotNone(response_future.traceback())
        self.assertIs(
            grpc.StatusCode.DEADLINE_EXCEEDED,
            response_future.exception().code(),
        )

    def testExpiredUnaryRequestStreamResponse(self):
        self._expired_unary_request_stream_response(
            unary_stream_multi_callable(self._channel)
        )

    def testExpiredUnaryRequestStreamResponseNonBlocking(self):
        self._expired_unary_request_stream_response(
            unary_stream_non_blocking_multi_callable(self._channel)
        )


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)



# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Download and unzip the target file to the destination."""

from __future__ import print_function

import os
import sys
import tempfile
import zipfile

import requests


def main():
    if len(sys.argv) != 3:
        print("Usage: python download_and_unzip.py [zipfile-url] [destination]")
        sys.exit(1)
    download_url = sys.argv[1]
    destination = sys.argv[2]

    with tempfile.TemporaryFile() as tmp_file:
        r = requests.get(download_url)
        if r.status_code != requests.codes.ok:
            print(
                'Download %s failed with [%d] "%s"'
                % (download_url, r.status_code, r.text())
            )
            sys.exit(1)
        else:
            tmp_file.write(r.content)
            print("Successfully downloaded from %s", download_url)
        with zipfile.ZipFile(tmp_file, "r") as target_zip_file:
            target_zip_file.extractall(destination)
        print("Successfully unzip to %s" % destination)


if __name__ == "__main__":
    main()



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import struct

import h2
import h2.connection
import messages_pb2
import twisted
import twisted.internet
import twisted.internet.protocol

_READ_CHUNK_SIZE = 16384
_GRPC_HEADER_SIZE = 5
_MIN_SETTINGS_MAX_FRAME_SIZE = 16384


class H2ProtocolBaseServer(twisted.internet.protocol.Protocol):
    def __init__(self):
        self._conn = h2.connection.H2Connection(client_side=False)
        self._recv_buffer = {}
        self._handlers = {}
        self._handlers["ConnectionMade"] = self.on_connection_made_default
        self._handlers["DataReceived"] = self.on_data_received_default
        self._handlers["WindowUpdated"] = self.on_window_update_default
        self._handlers["RequestReceived"] = self.on_request_received_default
        self._handlers["SendDone"] = self.on_send_done_default
        self._handlers["ConnectionLost"] = self.on_connection_lost
        self._handlers["PingAcknowledged"] = self.on_ping_acknowledged_default
        self._stream_status = {}
        self._send_remaining = {}
        self._outstanding_pings = 0

    def set_handlers(self, handlers):
        self._handlers = handlers

    def connectionMade(self):
        self._handlers["ConnectionMade"]()

    def connectionLost(self, reason):
        self._handlers["ConnectionLost"](reason)

    def on_connection_made_default(self):
        logging.info("Connection Made")
        self._conn.initiate_connection()
        self.transport.setTcpNoDelay(True)
        self.transport.write(self._conn.data_to_send())

    def on_connection_lost(self, reason):
        logging.info("Disconnected %s" % reason)

    def dataReceived(self, data):
        try:
            events = self._conn.receive_data(data)
        except h2.exceptions.ProtocolError:
            # this try/except block catches exceptions due to race between sending
            # GOAWAY and processing a response in flight.
            return
        if self._conn.data_to_send:
            self.transport.write(self._conn.data_to_send())
        for event in events:
            if isinstance(
                event, h2.events.RequestReceived
            ) and self._handlers.has_key("RequestReceived"):
                logging.info(
                    "RequestReceived Event for stream: %d" % event.stream_id
                )
                self._handlers["RequestReceived"](event)
            elif isinstance(
                event, h2.events.DataReceived
            ) and self._handlers.has_key("DataReceived"):
                logging.info(
                    "DataReceived Event for stream: %d" % event.stream_id
                )
                self._handlers["DataReceived"](event)
            elif isinstance(
                event, h2.events.WindowUpdated
            ) and self._handlers.has_key("WindowUpdated"):
                logging.info(
                    "WindowUpdated Event for stream: %d" % event.stream_id
                )
                self._handlers["WindowUpdated"](event)
            elif isinstance(
                event, h2.events.PingAcknowledged
            ) and self._handlers.has_key("PingAcknowledged"):
                logging.info("PingAcknowledged Event")
                self._handlers["PingAcknowledged"](event)
        self.transport.write(self._conn.data_to_send())

    def on_ping_acknowledged_default(self, event):
        logging.info("ping acknowledged")
        self._outstanding_pings -= 1

    def on_data_received_default(self, event):
        self._conn.acknowledge_received_data(len(event.data), event.stream_id)
        self._recv_buffer[event.stream_id] += event.data

    def on_request_received_default(self, event):
        self._recv_buffer[event.stream_id] = ""
        self._stream_id = event.stream_id
        self._stream_status[event.stream_id] = True
        self._conn.send_headers(
            stream_id=event.stream_id,
            headers=[
                (":status", "200"),
                ("content-type", "application/grpc"),
                ("grpc-encoding", "identity"),
                ("grpc-accept-encoding", "identity,deflate,gzip"),
            ],
        )
        self.transport.write(self._conn.data_to_send())

    def on_window_update_default(
        self, _, pad_length=None, read_chunk_size=_READ_CHUNK_SIZE
    ):
        # try to resume sending on all active streams (update might be for connection)
        for stream_id in self._send_remaining:
            self.default_send(
                stream_id,
                pad_length=pad_length,
                read_chunk_size=read_chunk_size,
            )

    def send_reset_stream(self):
        self._conn.reset_stream(self._stream_id)
        self.transport.write(self._conn.data_to_send())

    def setup_send(
        self,
        data_to_send,
        stream_id,
        pad_length=None,
        read_chunk_size=_READ_CHUNK_SIZE,
    ):
        logging.info("Setting up data to send for stream_id: %d" % stream_id)
        self._send_remaining[stream_id] = len(data_to_send)
        self._send_offset = 0
        self._data_to_send = data_to_send
        self.default_send(
            stream_id, pad_length=pad_length, read_chunk_size=read_chunk_size
        )

    def default_send(
        self, stream_id, pad_length=None, read_chunk_size=_READ_CHUNK_SIZE
    ):
        if not self._send_remaining.has_key(stream_id):
            # not setup to send data yet
            return

        while self._send_remaining[stream_id] > 0:
            lfcw = self._conn.local_flow_control_window(stream_id)
            padding_bytes = pad_length + 1 if pad_length is not None else 0
            if lfcw - padding_bytes <= 0:
                logging.info(
                    "Stream %d. lfcw: %d. padding bytes: %d. not enough"
                    " quota yet" % (stream_id, lfcw, padding_bytes)
                )
                break
            chunk_size = min(lfcw - padding_bytes, read_chunk_size)
            bytes_to_send = min(chunk_size, self._send_remaining[stream_id])
            logging.info(
                "flow_control_window = %d. sending [%d:%d] stream_id %d."
                " includes %d total padding bytes"
                % (
                    lfcw,
                    self._send_offset,
                    self._send_offset + bytes_to_send + padding_bytes,
                    stream_id,
                    padding_bytes,
                )
            )
            # The receiver might allow sending frames larger than the http2 minimum
            # max frame size (16384), but this test should never send more than 16384
            # for simplicity (which is always legal).
            if bytes_to_send + padding_bytes > _MIN_SETTINGS_MAX_FRAME_SIZE:
                raise ValueError(
                    "overload: sending %d" % (bytes_to_send + padding_bytes)
                )
            data = self._data_to_send[
                self._send_offset : self._send_offset + bytes_to_send
            ]
            try:
                self._conn.send_data(
                    stream_id, data, end_stream=False, pad_length=pad_length
                )
            except h2.exceptions.ProtocolError:
                logging.info("Stream %d is closed" % stream_id)
                break
            self._send_remaining[stream_id] -= bytes_to_send
            self._send_offset += bytes_to_send
            if self._send_remaining[stream_id] == 0:
                self._handlers["SendDone"](stream_id)

    def default_ping(self):
        logging.info("sending ping")
        self._outstanding_pings += 1
        self._conn.ping(b"\x00" * 8)
        self.transport.write(self._conn.data_to_send())

    def on_send_done_default(self, stream_id):
        if self._stream_status[stream_id]:
            self._stream_status[stream_id] = False
            self.default_send_trailer(stream_id)
        else:
            logging.error("Stream %d is already closed" % stream_id)

    def default_send_trailer(self, stream_id):
        logging.info("Sending trailer for stream id %d" % stream_id)
        self._conn.send_headers(
            stream_id, headers=[("grpc-status", "0")], end_stream=True
        )
        self.transport.write(self._conn.data_to_send())

    @staticmethod
    def default_response_data(response_size):
        sresp = messages_pb2.SimpleResponse()
        sresp.payload.body = b"\x00" * response_size
        serialized_resp_proto = sresp.SerializeToString()
        response_data = (
            b"\x00"
            + struct.pack("i", len(serialized_resp_proto))[::-1]
            + serialized_resp_proto
        )
        return response_data

    def parse_received_data(self, stream_id):
        """returns a grpc framed string of bytes containing response proto of the size
        asked in request"""
        recv_buffer = self._recv_buffer[stream_id]
        grpc_msg_size = struct.unpack("i", recv_buffer[1:5][::-1])[0]
        if len(recv_buffer) != _GRPC_HEADER_SIZE + grpc_msg_size:
            return None
        req_proto_str = recv_buffer[5 : 5 + grpc_msg_size]
        sr = messages_pb2.SimpleRequest()
        sr.ParseFromString(req_proto_str)
        logging.info("Parsed simple request for stream %d" % stream_id)
        return sr



# Copyright 2026 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# absl Utilities

cdef extern from "absl/log/initialize.h" namespace "absl":
  void InitializeLog() nogil

cdef extern from "absl/base/log_severity.h" namespace "absl":
  cpdef enum class LogSeverityAtLeast(int):
    kInfo = 0
    kWarning = 1
    kError = 2
    kFatal = 3

cdef extern from "absl/log/globals.h" namespace "absl":
  void SetStderrThreshold(LogSeverityAtLeast threshold) nogil



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests Face interface compliance of the gRPC Python Beta API."""

import threading
import unittest

from grpc.beta import implementations
from grpc.beta import interfaces
from grpc.framework.common import cardinality
from grpc.framework.interfaces.face import utilities

from tests.unit import resources
from tests.unit.beta import test_utilities
from tests.unit.framework.common import test_constants

_SERVER_HOST_OVERRIDE = "foo.test.google.fr"

_PER_RPC_CREDENTIALS_METADATA_KEY = b"my-call-credentials-metadata-key"
_PER_RPC_CREDENTIALS_METADATA_VALUE = b"my-call-credentials-metadata-value"

_GROUP = "group"
_UNARY_UNARY = "unary-unary"
_UNARY_STREAM = "unary-stream"
_STREAM_UNARY = "stream-unary"
_STREAM_STREAM = "stream-stream"

_REQUEST = b"abc"
_RESPONSE = b"123"


class _Servicer:
    def __init__(self):
        self._condition = threading.Condition()
        self._peer = None
        self._serviced = False

    def unary_unary(self, request, context):
        with self._condition:
            self._request = request
            self._peer = context.protocol_context().peer()
            self._invocation_metadata = context.invocation_metadata()
            context.protocol_context().disable_next_response_compression()
            self._serviced = True
            self._condition.notify_all()
            return _RESPONSE

    def unary_stream(self, request, context):
        with self._condition:
            self._request = request
            self._peer = context.protocol_context().peer()
            self._invocation_metadata = context.invocation_metadata()
            context.protocol_context().disable_next_response_compression()
            self._serviced = True
            self._condition.notify_all()
            return
            yield  # pylint: disable=unreachable

    def stream_unary(self, request_iterator, context):
        for request in request_iterator:
            self._request = request
        with self._condition:
            self._peer = context.protocol_context().peer()
            self._invocation_metadata = context.invocation_metadata()
            context.protocol_context().disable_next_response_compression()
            self._serviced = True
            self._condition.notify_all()
            return _RESPONSE

    def stream_stream(self, request_iterator, context):
        for request in request_iterator:
            with self._condition:
                self._peer = context.protocol_context().peer()
                context.protocol_context().disable_next_response_compression()
                yield _RESPONSE
        with self._condition:
            self._invocation_metadata = context.invocation_metadata()
            self._serviced = True
            self._condition.notify_all()

    def peer(self):
        with self._condition:
            return self._peer

    def block_until_serviced(self):
        with self._condition:
            while not self._serviced:
                self._condition.wait()


class _BlockingIterator:
    def __init__(self, upstream):
        self._condition = threading.Condition()
        self._upstream = upstream
        self._allowed = []

    def __iter__(self):
        return self

    def __next__(self):
        return self.next()

    def next(self):
        with self._condition:
            while True:
                if self._allowed is None:
                    raise StopIteration()
                elif self._allowed:
                    return self._allowed.pop(0)
                else:
                    self._condition.wait()

    def allow(self):
        with self._condition:
            try:
                self._allowed.append(next(self._upstream))
            except StopIteration:
                self._allowed = None
            self._condition.notify_all()


def _metadata_plugin(context, callback):
    callback(
        [
            (
                _PER_RPC_CREDENTIALS_METADATA_KEY,
                _PER_RPC_CREDENTIALS_METADATA_VALUE,
            )
        ],
        None,
    )


class BetaFeaturesTest(unittest.TestCase):
    def setUp(self):
        self._servicer = _Servicer()
        method_implementations = {
            (_GROUP, _UNARY_UNARY): utilities.unary_unary_inline(
                self._servicer.unary_unary
            ),
            (_GROUP, _UNARY_STREAM): utilities.unary_stream_inline(
                self._servicer.unary_stream
            ),
            (_GROUP, _STREAM_UNARY): utilities.stream_unary_inline(
                self._servicer.stream_unary
            ),
            (_GROUP, _STREAM_STREAM): utilities.stream_stream_inline(
                self._servicer.stream_stream
            ),
        }

        cardinalities = {
            _UNARY_UNARY: cardinality.Cardinality.UNARY_UNARY,
            _UNARY_STREAM: cardinality.Cardinality.UNARY_STREAM,
            _STREAM_UNARY: cardinality.Cardinality.STREAM_UNARY,
            _STREAM_STREAM: cardinality.Cardinality.STREAM_STREAM,
        }

        server_options = implementations.server_options(
            thread_pool_size=test_constants.POOL_SIZE
        )
        self._server = implementations.server(
            method_implementations, options=server_options
        )
        server_credentials = implementations.ssl_server_credentials(
            [
                (
                    resources.private_key(),
                    resources.certificate_chain(),
                ),
            ]
        )
        port = self._server.add_secure_port("[::]:0", server_credentials)
        self._server.start()
        self._channel_credentials = implementations.ssl_channel_credentials(
            resources.test_root_certificates()
        )
        self._call_credentials = implementations.metadata_call_credentials(
            _metadata_plugin
        )
        channel = test_utilities.not_really_secure_channel(
            "localhost", port, self._channel_credentials, _SERVER_HOST_OVERRIDE
        )
        stub_options = implementations.stub_options(
            thread_pool_size=test_constants.POOL_SIZE
        )
        self._dynamic_stub = implementations.dynamic_stub(
            channel, _GROUP, cardinalities, options=stub_options
        )

    def tearDown(self):
        self._dynamic_stub = None
        self._server.stop(test_constants.SHORT_TIMEOUT).wait()

    def test_unary_unary(self):
        call_options = interfaces.grpc_call_options(
            disable_compression=True, credentials=self._call_credentials
        )
        response = getattr(self._dynamic_stub, _UNARY_UNARY)(
            _REQUEST, test_constants.LONG_TIMEOUT, protocol_options=call_options
        )
        self.assertEqual(_RESPONSE, response)
        self.assertIsNotNone(self._servicer.peer())
        invocation_metadata = [
            (metadatum.key, metadatum.value)
            for metadatum in self._servicer._invocation_metadata
        ]
        self.assertIn(
            (
                _PER_RPC_CREDENTIALS_METADATA_KEY,
                _PER_RPC_CREDENTIALS_METADATA_VALUE,
            ),
            invocation_metadata,
        )

    def test_unary_stream(self):
        call_options = interfaces.grpc_call_options(
            disable_compression=True, credentials=self._call_credentials
        )
        response_iterator = getattr(self._dynamic_stub, _UNARY_STREAM)(
            _REQUEST, test_constants.LONG_TIMEOUT, protocol_options=call_options
        )
        self._servicer.block_until_serviced()
        self.assertIsNotNone(self._servicer.peer())
        invocation_metadata = [
            (metadatum.key, metadatum.value)
            for metadatum in self._servicer._invocation_metadata
        ]
        self.assertIn(
            (
                _PER_RPC_CREDENTIALS_METADATA_KEY,
                _PER_RPC_CREDENTIALS_METADATA_VALUE,
            ),
            invocation_metadata,
        )

    def test_stream_unary(self):
        call_options = interfaces.grpc_call_options(
            credentials=self._call_credentials
        )
        request_iterator = _BlockingIterator(iter((_REQUEST,)))
        response_future = getattr(self._dynamic_stub, _STREAM_UNARY).future(
            request_iterator,
            test_constants.LONG_TIMEOUT,
            protocol_options=call_options,
        )
        response_future.protocol_context().disable_next_request_compression()
        request_iterator.allow()
        response_future.protocol_context().disable_next_request_compression()
        request_iterator.allow()
        self._servicer.block_until_serviced()
        self.assertIsNotNone(self._servicer.peer())
        self.assertEqual(_RESPONSE, response_future.result())
        invocation_metadata = [
            (metadatum.key, metadatum.value)
            for metadatum in self._servicer._invocation_metadata
        ]
        self.assertIn(
            (
                _PER_RPC_CREDENTIALS_METADATA_KEY,
                _PER_RPC_CREDENTIALS_METADATA_VALUE,
            ),
            invocation_metadata,
        )

    def test_stream_stream(self):
        call_options = interfaces.grpc_call_options(
            credentials=self._call_credentials
        )
        request_iterator = _BlockingIterator(iter((_REQUEST,)))
        response_iterator = getattr(self._dynamic_stub, _STREAM_STREAM)(
            request_iterator,
            test_constants.SHORT_TIMEOUT,
            protocol_options=call_options,
        )
        response_iterator.protocol_context().disable_next_request_compression()
        request_iterator.allow()
        response = next(response_iterator)
        response_iterator.protocol_context().disable_next_request_compression()
        request_iterator.allow()
        self._servicer.block_until_serviced()
        self.assertIsNotNone(self._servicer.peer())
        self.assertEqual(_RESPONSE, response)
        invocation_metadata = [
            (metadatum.key, metadatum.value)
            for metadatum in self._servicer._invocation_metadata
        ]
        self.assertIn(
            (
                _PER_RPC_CREDENTIALS_METADATA_KEY,
                _PER_RPC_CREDENTIALS_METADATA_VALUE,
            ),
            invocation_metadata,
        )


class ContextManagementAndLifecycleTest(unittest.TestCase):
    def setUp(self):
        self._servicer = _Servicer()
        self._method_implementations = {
            (_GROUP, _UNARY_UNARY): utilities.unary_unary_inline(
                self._servicer.unary_unary
            ),
            (_GROUP, _UNARY_STREAM): utilities.unary_stream_inline(
                self._servicer.unary_stream
            ),
            (_GROUP, _STREAM_UNARY): utilities.stream_unary_inline(
                self._servicer.stream_unary
            ),
            (_GROUP, _STREAM_STREAM): utilities.stream_stream_inline(
                self._servicer.stream_stream
            ),
        }

        self._cardinalities = {
            _UNARY_UNARY: cardinality.Cardinality.UNARY_UNARY,
            _UNARY_STREAM: cardinality.Cardinality.UNARY_STREAM,
            _STREAM_UNARY: cardinality.Cardinality.STREAM_UNARY,
            _STREAM_STREAM: cardinality.Cardinality.STREAM_STREAM,
        }

        self._server_options = implementations.server_options(
            thread_pool_size=test_constants.POOL_SIZE
        )
        self._server_credentials = implementations.ssl_server_credentials(
            [
                (
                    resources.private_key(),
                    resources.certificate_chain(),
                ),
            ]
        )
        self._channel_credentials = implementations.ssl_channel_credentials(
            resources.test_root_certificates()
        )
        self._stub_options = implementations.stub_options(
            thread_pool_size=test_constants.POOL_SIZE
        )

    def test_stub_context(self):
        server = implementations.server(
            self._method_implementations, options=self._server_options
        )
        port = server.add_secure_port("[::]:0", self._server_credentials)
        server.start()

        channel = test_utilities.not_really_secure_channel(
            "localhost", port, self._channel_credentials, _SERVER_HOST_OVERRIDE
        )
        dynamic_stub = implementations.dynamic_stub(
            channel, _GROUP, self._cardinalities, options=self._stub_options
        )
        for _ in range(100):
            with dynamic_stub:
                pass
        for _ in range(10):
            with dynamic_stub:
                call_options = interfaces.grpc_call_options(
                    disable_compression=True
                )
                response = getattr(dynamic_stub, _UNARY_UNARY)(
                    _REQUEST,
                    test_constants.LONG_TIMEOUT,
                    protocol_options=call_options,
                )
                self.assertEqual(_RESPONSE, response)
                self.assertIsNotNone(self._servicer.peer())

        server.stop(test_constants.SHORT_TIMEOUT).wait()

    def test_server_lifecycle(self):
        for _ in range(100):
            server = implementations.server(
                self._method_implementations, options=self._server_options
            )
            port = server.add_secure_port("[::]:0", self._server_credentials)
            server.start()
            server.stop(test_constants.SHORT_TIMEOUT).wait()
        for _ in range(100):
            server = implementations.server(
                self._method_implementations, options=self._server_options
            )
            server.add_secure_port("[::]:0", self._server_credentials)
            server.add_insecure_port("[::]:0")
            with server:
                server.stop(test_constants.SHORT_TIMEOUT)
            server.stop(test_constants.SHORT_TIMEOUT)


if __name__ == "__main__":
    unittest.main(verbosity=2)



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function

import argparse
import json
import uuid

from apiclient import discovery
from apiclient.errors import HttpError
import httplib2
from oauth2client.client import GoogleCredentials

# 30 days in milliseconds
_EXPIRATION_MS = 30 * 24 * 60 * 60 * 1000
NUM_RETRIES = 3


def create_big_query():
    """Authenticates with cloud platform and gets a BiqQuery service object"""
    creds = GoogleCredentials.get_application_default()
    return discovery.build(
        "bigquery", "v2", credentials=creds, cache_discovery=False
    )


def create_dataset(biq_query, project_id, dataset_id):
    is_success = True
    body = {
        "datasetReference": {"projectId": project_id, "datasetId": dataset_id}
    }

    try:
        dataset_req = biq_query.datasets().insert(
            projectId=project_id, body=body
        )
        dataset_req.execute(num_retries=NUM_RETRIES)
    except HttpError as http_error:
        if http_error.resp.status == 409:
            print("Warning: The dataset %s already exists" % dataset_id)
        else:
            # Note: For more debugging info, print "http_error.content"
            print(
                "Error in creating dataset: %s. Err: %s"
                % (dataset_id, http_error)
            )
            is_success = False
    return is_success


def create_table(
    big_query, project_id, dataset_id, table_id, table_schema, description
):
    fields = [
        {
            "name": field_name,
            "type": field_type,
            "description": field_description,
        }
        for (field_name, field_type, field_description) in table_schema
    ]
    return create_table2(
        big_query, project_id, dataset_id, table_id, fields, description
    )


def create_partitioned_table(
    big_query,
    project_id,
    dataset_id,
    table_id,
    table_schema,
    description,
    partition_type="DAY",
    expiration_ms=_EXPIRATION_MS,
):
    """Creates a partitioned table. By default, a date-paritioned table is created with
    each partition lasting 30 days after it was last modified.
    """
    fields = [
        {
            "name": field_name,
            "type": field_type,
            "description": field_description,
        }
        for (field_name, field_type, field_description) in table_schema
    ]
    return create_table2(
        big_query,
        project_id,
        dataset_id,
        table_id,
        fields,
        description,
        partition_type,
        expiration_ms,
    )


def create_table2(
    big_query,
    project_id,
    dataset_id,
    table_id,
    fields_schema,
    description,
    partition_type=None,
    expiration_ms=None,
):
    is_success = True

    body = {
        "description": description,
        "schema": {"fields": fields_schema},
        "tableReference": {
            "datasetId": dataset_id,
            "projectId": project_id,
            "tableId": table_id,
        },
    }

    if partition_type and expiration_ms:
        body["timePartitioning"] = {
            "type": partition_type,
            "expirationMs": expiration_ms,
        }

    try:
        table_req = big_query.tables().insert(
            projectId=project_id, datasetId=dataset_id, body=body
        )
        res = table_req.execute(num_retries=NUM_RETRIES)
        print('Successfully created %s "%s"' % (res["kind"], res["id"]))
    except HttpError as http_error:
        if http_error.resp.status == 409:
            print("Warning: Table %s already exists" % table_id)
        else:
            print(
                "Error in creating table: %s. Err: %s" % (table_id, http_error)
            )
            is_success = False
    return is_success


def patch_table(big_query, project_id, dataset_id, table_id, fields_schema):
    is_success = True

    body = {
        "schema": {"fields": fields_schema},
        "tableReference": {
            "datasetId": dataset_id,
            "projectId": project_id,
            "tableId": table_id,
        },
    }

    try:
        table_req = big_query.tables().patch(
            projectId=project_id,
            datasetId=dataset_id,
            tableId=table_id,
            body=body,
        )
        res = table_req.execute(num_retries=NUM_RETRIES)
        print('Successfully patched %s "%s"' % (res["kind"], res["id"]))
    except HttpError as http_error:
        print("Error in creating table: %s. Err: %s" % (table_id, http_error))
        is_success = False
    return is_success


def insert_rows(big_query, project_id, dataset_id, table_id, rows_list):
    is_success = True
    body = {"rows": rows_list}
    try:
        insert_req = big_query.tabledata().insertAll(
            projectId=project_id,
            datasetId=dataset_id,
            tableId=table_id,
            body=body,
        )
        res = insert_req.execute(num_retries=NUM_RETRIES)
        if res.get("insertErrors", None):
            print("Error inserting rows! Response: %s" % res)
            is_success = False
    except HttpError as http_error:
        print("Error inserting rows to the table %s" % table_id)
        print("Error message: %s" % http_error)
        is_success = False

    return is_success


def sync_query_job(big_query, project_id, query, timeout=5000):
    query_data = {"query": query, "timeoutMs": timeout}
    query_job = None
    try:
        query_job = (
            big_query.jobs()
            .query(projectId=project_id, body=query_data)
            .execute(num_retries=NUM_RETRIES)
        )
    except HttpError as http_error:
        print("Query execute job failed with error: %s" % http_error)
        print(http_error.content)
    return query_job

    # List of (column name, column type, description) tuples


def make_row(unique_row_id, row_values_dict):
    """row_values_dict is a dictionary of column name and column value."""
    return {"insertId": unique_row_id, "json": row_values_dict}



# Copyright 2024 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys

import setuptools

# Manually insert the source directory into the Python path for local module
# imports to succeed
sys.path.insert(0, os.path.abspath("."))

import grpc_version
import python_version

INSTALL_REQUIRES = (
    "opentelemetry-sdk>=1.25.0",
    "opentelemetry-resourcedetector-gcp>=1.6.0a0",
    "grpcio=={version}".format(version=grpc_version.VERSION),
    "protobuf>=6.33.5,<8.0.0",
)

if __name__ == "__main__":
    setuptools.setup(
        python_requires=f">={python_version.MIN_PYTHON_VERSION}",
        install_requires=INSTALL_REQUIRES,
    )
